@@ -1,4 +1,4 @@
-binomial.CARanova <- function(formula, data=NULL, trials, W, interaction=TRUE, burnin, n.sample, thin=1,  prior.mean.beta=NULL, prior.var.beta=NULL, prior.tau2=NULL, verbose=TRUE)
+binomial.CARanova <- function(formula, data=NULL, trials, W, interaction=TRUE, burnin, n.sample, thin=1,  prior.mean.beta=NULL, prior.var.beta=NULL, prior.tau2=NULL, fix.rho.S=FALSE, rho.S=NULL, fix.rho.T=FALSE, rho.T=NULL,verbose=TRUE)
 {
     #### Check on the verbose option
     if(is.null(verbose)) verbose=TRUE     
@@ -65,9 +65,25 @@ binomial.CARanova <- function(formula, data=NULL, trials, W, interaction=TRUE, b
     }
     
     
-#### Response variable
-Y <- model.response(frame)
+    #### Format and check the neighbourhood matrix W
+    if(!is.matrix(W)) stop("W is not a matrix.", call.=FALSE)
+    K <- nrow(W)
+    if(ncol(W)!= K) stop("W has the wrong number of columns.", call.=FALSE)
+    if(nrow(W)!= K) stop("W has the wrong number of rows.", call.=FALSE)
+    if(floor(N.all/K)!=ceiling(N.all/K)) stop("The number of spatial areas is not a multiple of the number of data points.", call.=FALSE)
+    N <- N.all / K
+    if(sum(is.na(W))>0) stop("W has missing 'NA' values.", call.=FALSE)
+    if(!is.numeric(W)) stop("W has non-numeric values.", call.=FALSE)
+    if(min(W)<0) stop("W has negative elements.", call.=FALSE)
+    if(sum(W!=t(W))>0) stop("W is not symmetric.", call.=FALSE)
+    if(min(apply(W, 1, sum))==0) stop("W has some areas with no neighbours (one of the row sums equals zero).", call.=FALSE)    
     
+    
+    #### Response variable
+Y <- model.response(frame)
+Y <- as.numeric(Y)
+failures <- trials - Y
+
     ## Check for errors
     if(sum(is.na(trials))>0) stop("the numbers of trials has missing 'NA' values.", call.=FALSE)
     if(!is.numeric(trials)) stop("the numbers of trials has non-numeric values.", call.=FALSE)
@@ -75,16 +91,25 @@ Y <- model.response(frame)
     if(int.check > 0) stop("the numbers of trials has non-integer values.", call.=FALSE)
     if(min(trials)<=0) stop("the numbers of trials has zero or negative values.", call.=FALSE)
     
-    if(sum(is.na(Y))>0) stop("the response has missing 'NA' values.", call.=FALSE)
+    ## Create the missing value indicator
+    which.miss <- as.numeric(!is.na(Y))
+    which.miss.mat <- matrix(which.miss, nrow=K, ncol=N, byrow=FALSE)
+    n.miss <- N.all - sum(which.miss)
+    Y.miss <- Y
+    Y.miss[which.miss==0] <- median(Y, na.rm=TRUE)
+    failures.miss <- failures
+    failures.miss[which.miss==0] <- median(failures, na.rm=TRUE)
+    
     if(!is.numeric(Y)) stop("the response variable has non-numeric values.", call.=FALSE)
-    int.check <- N.all-sum(ceiling(Y)==floor(Y))
-    if(int.check > 0) stop("the respons variable has non-integer values.", call.=FALSE)
-    if(min(Y)<0) stop("the response variable has negative values.", call.=FALSE)
-    if(sum(Y>trials)>0) stop("the response variable has larger values that the numbers of trials.", call.=FALSE)
-Y <- as.numeric(Y)
-failures <- trials - Y
-    
+    int.check <- N.all- n.miss - sum(ceiling(Y)==floor(Y), na.rm=TRUE)
+    if(int.check > 0) stop("the response variable has non-integer values.", call.=FALSE)
+    if(min(Y, na.rm=TRUE)<0) stop("the response variable has negative values.", call.=FALSE)
+    if(sum(Y>trials, na.rm=TRUE)>0) stop("the response variable has larger values that the numbers of trials.", call.=FALSE)
+
     
+
+
+
 #### Offset variable
 offset <- try(model.offset(frame), silent=TRUE)
     if(class(offset)=="try-error")   stop("the offset is not numeric.", call.=FALSE)
@@ -94,18 +119,7 @@ offset <- try(model.offset(frame), silent=TRUE)
     
     
     
-    #### Format and check the neighbourhood matrix W
-    if(!is.matrix(W)) stop("W is not a matrix.", call.=FALSE)
-K <- nrow(W)
-    if(ncol(W)!= K) stop("W has the wrong number of columns.", call.=FALSE)
-    if(nrow(W)!= K) stop("W has the wrong number of rows.", call.=FALSE)
-    if(floor(N.all/K)!=ceiling(N.all/K)) stop("The number of spatial areas is not a multiple of the number of data points.", call.=FALSE)
-N <- N.all / K
-    if(sum(is.na(W))>0) stop("W has missing 'NA' values.", call.=FALSE)
-    if(!is.numeric(W)) stop("W has non-numeric values.", call.=FALSE)
-    if(min(W)<0) stop("W has negative elements.", call.=FALSE)
-    if(sum(W!=t(W))>0) stop("W is not symmetric.", call.=FALSE)
-    if(min(apply(W, 1, sum))==0) stop("W has some areas with no neighbours (one of the row sums equals zero).", call.=FALSE)    
+
     
     #### Checks on the interaction flag
     if(sum(interaction==c(TRUE, FALSE)) != 1) stop("interaction must be either TRUE or FALSE.", call.=FALSE)
@@ -115,27 +129,43 @@ N <- N.all / K
     
 #### Specify the initial parameter values
 dat <- cbind(Y, failures)
-beta <- glm(dat~X.standardised-1, offset=offset, family=binomial)$coefficients
+mod.glm <- glm(dat~X.standardised-1, offset=offset, family="quasibinomial")
+beta.mean <- mod.glm$coefficients
+beta.sd <- sqrt(diag(summary(mod.glm)$cov.scaled))
+beta <- rnorm(n=length(beta.mean), mean=beta.mean, sd=beta.sd)
+
+
 theta.hat <- Y / trials
 theta.hat[theta.hat==0] <- 0.01
 theta.hat[theta.hat==1] <- 0.99
-regression.vec <- X.standardised %*% beta
-res.temp <- log(theta.hat / (1 - theta.hat)) - regression.vec - offset
-phi <- rnorm(n=K, mean=0, sd = 2*sd(res.temp))
-delta <- rnorm(n=N, mean=0, sd = 2*sd(res.temp))
-tau2.phi <- runif(1, min=var(res.temp)/2, max=var(res.temp)*2)
-tau2.delta <- runif(1, min=var(res.temp)/2, max=var(res.temp)*2)
-rho <- runif(1)  
-lambda <- runif(1)  
+res.temp <- log(theta.hat / (1 - theta.hat)) - X.standardised %*% beta - offset
+res.sd <- sd(res.temp, na.rm=TRUE)/5
+phi <- rnorm(n=K, mean=0, sd = res.sd)
+delta <- rnorm(n=N, mean=0, sd = res.sd)
+tau2.phi <- var(phi)/10
+tau2.delta <- var(delta)/10
     if(interaction)
     {
-    gamma <- rnorm(n=(N*K), mean=0, sd = 2*sd(res.temp))
-    tau2.gamma <- runif(1, min=var(res.temp)/2, max=var(res.temp)*2) 
+    gamma <- rnorm(n=N.all, mean=0, sd = res.sd)
+    tau2.gamma <- var(gamma)/10
     }else
     {}
 
+    if(fix.rho.S)
+    {
+    rho <- rho.S
+    }else
+    {
+     rho <- runif(1)       
+    }
 
-    
+    if(fix.rho.T)
+    {
+    lambda <- rho.T
+    }else
+    {
+    lambda <- runif(1)       
+    }   
     
     #### Check and specify the priors
     ## Put in default priors
@@ -201,17 +231,34 @@ lambda <- runif(1)
     }         
     
     
+    ## Check for errors on rho and fix.rho
+    if(!is.logical(fix.rho.S)) stop("fix.rho.S is not logical.", call.=FALSE)   
+    if(fix.rho.S & is.null(rho.S)) stop("rho.S is fixed but an initial value was not set.", call.=FALSE)   
+    if(fix.rho.S & !is.numeric(rho.S) ) stop("rho.S is not numeric.", call.=FALSE)  
+    if(rho<0 ) stop("rho.S is outside the range [0, 1].", call.=FALSE)  
+    if(rho>1 ) stop("rho.S is outside the range [0, 1].", call.=FALSE)  
+    
+    ## Check for errors on rho and fix.rho
+    if(!is.logical(fix.rho.T)) stop("fix.rho.T is not logical.", call.=FALSE)   
+    if(fix.rho.T & is.null(rho.T)) stop("rho.T is fixed but an initial value was not set.", call.=FALSE)   
+    if(fix.rho.T & !is.numeric(rho.T) ) stop("rho.T is not numeric.", call.=FALSE)  
+    if(lambda<0 ) stop("rho.T is outside the range [0, 1].", call.=FALSE)  
+    if(lambda>1 ) stop("rho.T is outside the range [0, 1].", call.=FALSE)  
+    
     
 #### Set up matrices to store samples
 n.keep <- floor((n.sample - burnin)/thin)
 samples.beta <- array(NA, c(n.keep, p))
 samples.phi <- array(NA, c(n.keep, K))
 samples.delta <- array(NA, c(n.keep, N))
-samples.rho <- array(NA, c(n.keep, 1))
-samples.lambda <- array(NA, c(n.keep, 1))
+if(!fix.rho.S) samples.rho <- array(NA, c(n.keep, 1))
+if(!fix.rho.T) samples.lambda <- array(NA, c(n.keep, 1))
 samples.fitted <- array(NA, c(n.keep, N.all))
+samples.like <- array(NA, c(n.keep, N.all))
 samples.deviance <- array(NA, c(n.keep, 1))
-    
+if(n.miss>0) samples.Y <- array(NA, c(n.keep, n.miss))
+
+
     if(interaction)
     {
         samples.gamma <- array(NA, c(n.keep, N.all))
@@ -224,7 +271,9 @@ samples.deviance <- array(NA, c(n.keep, 1))
     }
     
     
-    
+
+
+
     #### Specify the Metropolis quantities
     accept.all <- rep(0,12)
     accept <- accept.all
@@ -248,7 +297,7 @@ samples.deviance <- array(NA, c(n.keep, 1))
     
     
     
-    #### Spatial quantities
+    #### .S quantities
     ## Create the triplet object
     W.triplet <- c(NA, NA, NA)
     for(i in 1:K)
@@ -278,16 +327,20 @@ samples.deviance <- array(NA, c(n.keep, 1))
     
     
     ## Create the determinant     
-    Wstar <- diag(apply(W,1,sum)) - W
-    Wstar.eigen <- eigen(Wstar)
-    Wstar.val <- Wstar.eigen$values
-    det.Q.W <-  0.5 * sum(log((rho * Wstar.val + (1-rho))))    
+    if(!fix.rho.S) 
+    {
+        Wstar <- diag(apply(W,1,sum)) - W
+        Wstar.eigen <- eigen(Wstar)
+        Wstar.val <- Wstar.eigen$values
+        det.Q.W <-  0.5 * sum(log((rho * Wstar.val + (1-rho))))     
+    }else
+    {}
     
     
     
     
-    #### Temporal quantities
-    ## Temporal neighbourhood matrix
+    #### .T quantities
+    ## .T neighbourhood matrix
     D <-array(0, c(N,N))
     for(i in 1:N)
     {
@@ -327,11 +380,14 @@ samples.deviance <- array(NA, c(n.keep, 1))
     
     
     ## Create the determinant     
-    Dstar <- diag(apply(D,1,sum)) - D
-    Dstar.eigen <- eigen(Dstar)
-    Dstar.val <- Dstar.eigen$values
-    det.Q.D <-  0.5 * sum(log((lambda * Dstar.val + (1-lambda))))    
-    
+    if(!fix.rho.T) 
+    {
+        Dstar <- diag(apply(D,1,sum)) - D
+        Dstar.eigen <- eigen(Dstar)
+        Dstar.val <- Dstar.eigen$values
+        det.Q.D <-  0.5 * sum(log((lambda * Dstar.val + (1-lambda))))    
+    }else
+    {}    
     
     #### Specify quantities that do not change
     offset.mat <- matrix(offset, nrow=K, ncol=N, byrow=FALSE) 
@@ -340,7 +396,11 @@ samples.deviance <- array(NA, c(n.keep, 1))
     Y.mat.trans <- t(Y.mat)
     failures.mat <- matrix(failures, nrow=K, ncol=N, byrow=FALSE)
     failures.mat.trans <- t(failures.mat)
-    failures.vec <- as.numeric(failures)
+    Y.mat.miss <- matrix(Y.miss, nrow=K, ncol=N, byrow=FALSE)
+    Y.mat.trans.miss <- t(Y.mat.miss)
+    failures.mat.miss <- matrix(failures.miss, nrow=K, ncol=N, byrow=FALSE)
+    failures.mat.trans.miss <- t(failures.mat.miss)
+    which.miss.mat.trans <- t(which.miss.mat)
     phi.mat <- matrix(rep(phi, N), byrow=F, nrow=K)
     delta.mat <- matrix(rep(delta, K), byrow=T, nrow=K)
     
@@ -353,13 +413,24 @@ samples.deviance <- array(NA, c(n.keep, 1))
     }
     
     
+    ## Check for islands
+    W.list<- mat2listw(W)
+    W.nb <- W.list$neighbours
+    W.islands <- n.comp.nb(W.nb)
+    islands <- W.islands$comp.id
+    n.islands <- max(W.islands$nc)
+    if(rho==1) tau2.phi.shape <- prior.tau2[1] + 0.5 * (K-n.islands)   
+    if(lambda==1) tau2.delta.shape <- prior.tau2[1] + 0.5 * (N-1)   
+
+    
+    
     ###########################
     #### Run the Bayesian model
     ###########################
     ## Start timer
     if(verbose)
     {
-        cat("Collecting", n.sample, "samples\n", sep = " ")
+        cat("Generating", n.sample, "samples\n", sep = " ")
         progressBar <- txtProgressBar(style = 3)
         percentage.points<-round((1:100/100)*n.sample)
     }else
@@ -380,7 +451,7 @@ samples.deviance <- array(NA, c(n.keep, 1))
         for(r in 1:n.beta.block)
         {
             proposal.beta[beta.beg[r]:beta.fin[r]] <- proposal[beta.beg[r]:beta.fin[r]]
-            prob <- binomialbetaupdate(X.standardised, N.all, p, beta, proposal.beta, offset.temp, Y, failures, prior.mean.beta, prior.var.beta)
+            prob <- binomialbetaupdate(X.standardised, N.all, p, beta, proposal.beta, offset.temp, Y.miss, failures.miss, prior.mean.beta, prior.var.beta, which.miss)
              if(prob > runif(1))
             {
                 beta[beta.beg[r]:beta.fin[r]] <- proposal.beta[beta.beg[r]:beta.fin[r]]
@@ -400,9 +471,16 @@ samples.deviance <- array(NA, c(n.keep, 1))
         ## Sample from phi
         ####################
         phi.offset <- offset.mat + regression.mat + delta.mat + gamma.mat
-        temp1 <- binomialcarupdate(W.triplet, W.begfin, W.triplet.sum, K, phi, tau2.phi,Y.mat, failures.mat, proposal.sd.phi, rho, phi.offset, N, rep(1,N))
+        temp1 <- binomialcarupdate(W.triplet, W.begfin, W.triplet.sum, K, phi, tau2.phi,Y.mat.miss, failures.mat.miss, proposal.sd.phi, rho, phi.offset, N, rep(1,N), which.miss.mat)
         phi <- temp1[[1]]
-        phi <- phi - mean(phi)
+        if(rho<1)
+        {
+            phi <- phi - mean(phi)
+        }
+        else
+        {
+            phi[which(islands==1)] <- phi[which(islands==1)] - mean(phi[which(islands==1)])   
+        }
         phi.mat <- matrix(rep(phi, N), byrow=F, nrow=K)    
         accept[3] <- accept[3] + temp1[[2]]
         accept[4] <- accept[4] + K  
@@ -413,7 +491,7 @@ samples.deviance <- array(NA, c(n.keep, 1))
         ## Sample from delta
         ####################
         delta.offset <- t(offset.mat + regression.mat + phi.mat + gamma.mat)
-        temp2 <- binomialcarupdate(D.triplet, D.begfin, D.triplet.sum, N, delta, tau2.delta, Y.mat.trans, failures.mat.trans, proposal.sd.delta, lambda, delta.offset, K, rep(1,K))
+        temp2 <- binomialcarupdate(D.triplet, D.begfin, D.triplet.sum, N, delta, tau2.delta, Y.mat.trans.miss, failures.mat.trans.miss, proposal.sd.delta, lambda, delta.offset, K, rep(1,K), which.miss.mat.trans)
         delta <- temp2[[1]]
         delta <- delta - mean(delta)
         delta.mat <- matrix(rep(delta, K), byrow=T, nrow=K)
@@ -429,7 +507,7 @@ samples.deviance <- array(NA, c(n.keep, 1))
             ####################
             gamma.offset <- offset.mat + regression.mat + phi.mat +  delta.mat
             gamma.offset.vec <- as.numeric(gamma.offset)
-            temp5 <- binomialindepupdate(N.all, gamma, tau2.gamma, Y, failures, proposal.sd.gamma, gamma.offset.vec)
+            temp5 <- binomialindepupdate(N.all, gamma, tau2.gamma, Y.miss, failures.miss, proposal.sd.gamma, gamma.offset.vec, which.miss)
             gamma <- temp5[[1]]
             gamma <- gamma - mean(gamma)
             gamma.mat <- matrix(gamma, byrow=F, nrow=K)
@@ -467,6 +545,8 @@ samples.deviance <- array(NA, c(n.keep, 1))
         ##################
         ## Sample from rho
         ##################
+        if(!fix.rho.S)
+        {
         proposal.rho <- rtrunc(n=1, spec="norm", a=0, b=1, mean=rho, sd=proposal.sd.rho)   
         temp3 <- quadform(W.triplet, W.triplet.sum, W.n.triplet, K, phi, phi, proposal.rho)
         det.Q.proposal <- 0.5 * sum(log((proposal.rho * Wstar.val + (1-proposal.rho))))              
@@ -484,12 +564,15 @@ samples.deviance <- array(NA, c(n.keep, 1))
         {
         }              
         accept[10] <- accept[10] + 1           
-        
+        }else
+        {}
         
         
         #####################
         ## Sample from lambda
         #####################
+        if(!fix.rho.T)
+        {
         proposal.lambda <- rtrunc(n=1, spec="norm", a=0, b=1, mean=lambda, sd=proposal.sd.lambda)   
         temp3 <- quadform(D.triplet, D.triplet.sum, D.n.triplet, N, delta, delta, proposal.lambda)
         det.Q.proposal <- 0.5 * sum(log((proposal.lambda * Dstar.val + (1-proposal.lambda))))              
@@ -507,7 +590,8 @@ samples.deviance <- array(NA, c(n.keep, 1))
         {
         }              
         accept[12] <- accept[12] + 1           
-        
+        }else
+        {}
         
         
         #########################
@@ -517,8 +601,9 @@ samples.deviance <- array(NA, c(n.keep, 1))
         prob <- exp(lp) / (1+exp(lp))
         fitted <- trials * prob
         deviance.all <- dbinom(x=Y, size=trials, prob=prob, log=TRUE)
-        deviance <- -2 * sum(deviance.all)     
-          
+        like <- exp(deviance.all)
+        deviance <- -2 * sum(deviance.all, na.rm=TRUE)     
+
         
         
         ###################
@@ -530,10 +615,12 @@ samples.deviance <- array(NA, c(n.keep, 1))
             samples.beta[ele, ] <- beta
             samples.phi[ele, ] <- phi
             samples.delta[ele, ] <- delta
-            samples.rho[ele, ] <- rho
-            samples.lambda[ele, ] <- lambda
+            if(!fix.rho.S) samples.rho[ele, ] <- rho
+            if(!fix.rho.T) samples.lambda[ele, ] <- lambda
             samples.deviance[ele, ] <- deviance
             samples.fitted[ele, ] <- fitted
+            samples.like[ele, ] <- like
+            if(n.miss>0) samples.Y[ele, ] <- rbinom(n=n.miss, size=trials[which.miss==0], prob=prob[which.miss==0])
             
             if(interaction)
             {
@@ -566,67 +653,69 @@ samples.deviance <- array(NA, c(n.keep, 1))
                 accept.gamma <- 50    
             }
             accept.rho <- 100 * accept[9] / accept[10]
+            if(is.na(accept.rho)) accept.rho <- 45
             accept.lambda <- 100 * accept[11] / accept[12]
+            if(is.na(accept.lambda)) accept.lambda <- 45
             accept.all <- accept.all + accept
             accept <- rep(0,12)
             
             #### beta tuning parameter
             if(accept.beta > 40)
             {
-                proposal.sd.beta <- 2 * proposal.sd.beta
+                proposal.sd.beta <- proposal.sd.beta + 0.1 * proposal.sd.beta
             }else if(accept.beta < 20)              
             {
-                proposal.sd.beta <- 0.5 * proposal.sd.beta
+                proposal.sd.beta <- proposal.sd.beta - 0.1 * proposal.sd.beta
             }else
             {
             }
             #### phi tuning parameter
             if(accept.phi > 50)
             {
-                proposal.sd.phi <- 2 * proposal.sd.phi
+                proposal.sd.phi <- proposal.sd.phi + 0.1 * proposal.sd.phi
             }else if(accept.phi < 40)              
             {
-                proposal.sd.phi <- 0.5 * proposal.sd.phi
+                proposal.sd.phi <- proposal.sd.phi - 0.1 * proposal.sd.phi
             }else
             {
             }      
             #### delta tuning parameter
             if(accept.delta > 50)
             {
-                proposal.sd.delta <- 2 * proposal.sd.delta
+                proposal.sd.delta <- proposal.sd.delta + 0.1 * proposal.sd.delta
             }else if(accept.delta < 40)              
             {
-                proposal.sd.delta <- 0.5 * proposal.sd.delta
+                proposal.sd.delta <- proposal.sd.delta - 0.1 * proposal.sd.delta
             }else
             {
             }
             #### gamma tuning parameter
             if(accept.gamma > 50)
             {
-                proposal.sd.gamma <- 2 * proposal.sd.gamma
+                proposal.sd.gamma <- proposal.sd.gamma + 0.1 * proposal.sd.gamma
             }else if(accept.gamma < 40)              
             {
-                proposal.sd.gamma <- 0.5 * proposal.sd.gamma
+                proposal.sd.gamma <- proposal.sd.gamma - 0.1 * proposal.sd.gamma
             }else
             {
             }
             #### rho tuning parameter
             if(accept.rho > 50)
             {
-                proposal.sd.rho <- min(2 * proposal.sd.rho, 0.5)
+                proposal.sd.rho <- min(proposal.sd.rho + 0.1 * proposal.sd.rho, 0.5)
             }else if(accept.rho < 40)              
             {
-                proposal.sd.rho <- 0.5 * proposal.sd.rho
+                proposal.sd.rho <- proposal.sd.rho - 0.1 * proposal.sd.rho
             }else
             {
             }
             #### lambda tuning parameter
             if(accept.lambda > 50)
             {
-                proposal.sd.lambda <- min(2 * proposal.sd.lambda, 0.5)
+                proposal.sd.lambda <- min(proposal.sd.lambda + 0.1 * proposal.sd.lambda, 0.5)
             }else if(accept.lambda < 40)              
             {
-                proposal.sd.lambda <- 0.5 * proposal.sd.lambda
+                proposal.sd.lambda <- proposal.sd.lambda - 0.1 * proposal.sd.lambda
             }else
             {
             }      
@@ -660,18 +749,31 @@ samples.deviance <- array(NA, c(n.keep, 1))
 accept.beta <- 100 * accept.all[1] / accept.all[2]
 accept.phi <- 100 * accept.all[3] / accept.all[4]
 accept.delta <- 100 * accept.all[5] / accept.all[6]
-accept.rho <- 100 * accept.all[9] / accept.all[10]
-accept.lambda <- 100 * accept.all[11] / accept.all[12]
-    
+    if(!fix.rho.S)
+    {
+    accept.rho <- 100 * accept.all[9] / accept.all[10]
+    }else
+    {
+    accept.rho <- NA    
+    }
+
+    if(!fix.rho.T)
+    {
+    accept.lambda <- 100 * accept.all[11] / accept.all[12]
+    }else
+    {
+    accept.lambda <- NA    
+    }
+
     if(interaction)
     {
     accept.gamma <- 100 * accept.all[7] / accept.all[8]
     accept.final <- c(accept.beta, accept.phi, accept.delta, accept.gamma, accept.rho, accept.lambda)
-    names(accept.final) <- c("beta", "phi", "delta", "gamma", "rho", "lambda")
+    names(accept.final) <- c("beta", "phi", "delta", "gamma", "rho.S", "rho.T")
     }else
     {
     accept.final <- c(accept.beta, accept.phi, accept.delta, accept.rho, accept.lambda)
-    names(accept.final) <- c("beta", "phi", "delta", "rho", "lambda")        
+    names(accept.final) <- c("beta", "phi", "delta", "rho.S", "rho.T")        
     }
     
     
@@ -694,18 +796,24 @@ regression.mat <- matrix(X.standardised %*% median.beta, nrow=K, ncol=N, byrow=F
     
 median.prob <- exp(lp.median)  / (1 + exp(lp.median))
 fitted.median <- trials * median.prob
-deviance.fitted <- -2 * sum(dbinom(x=Y, size=trials, prob=median.prob, log=TRUE))
+deviance.fitted <- -2 * sum(dbinom(x=Y, size=trials, prob=median.prob, log=TRUE), na.rm=TRUE)
 p.d <- median(samples.deviance) - deviance.fitted
 DIC <- 2 * median(samples.deviance) - deviance.fitted     
 
 
+#### Watanabe-Akaike Information Criterion (WAIC)
+LPPD <- sum(log(apply(samples.like,2,mean)), na.rm=TRUE)
+p.w <- sum(apply(log(samples.like),2,var), na.rm=TRUE)
+WAIC <- -2 * (LPPD - p.w)
+
+
 ## Compute the LMPL
 CPO <- rep(NA, N.all)
     for(j in 1:N.all)
     {
     CPO[j] <- 1/median((1 / dbinom(x=Y[j], size=trials[j], prob=(samples.fitted[ ,j] / trials[j]))))    
     }
-LMPL <- sum(log(CPO))  
+LMPL <- sum(log(CPO), na.rm=TRUE)  
     
     
 ## Create the Fitted values
@@ -756,26 +864,58 @@ if(interaction)
     summary.hyper[1,1:3] <- quantile(samples.tau2[ ,1], c(0.5, 0.025, 0.975))
     summary.hyper[2,1:3] <- quantile(samples.tau2[ ,2], c(0.5, 0.025, 0.975))
     summary.hyper[3,1:3] <- quantile(samples.tau2[ ,3], c(0.5, 0.025, 0.975))
-    summary.hyper[4,1:3] <- quantile(samples.rho, c(0.5, 0.025, 0.975))
-    summary.hyper[5,1:3] <- quantile(samples.lambda, c(0.5, 0.025, 0.975))
-    rownames(summary.hyper) <- c("tau2.phi", "tau2.delta", "tau2.gamma", "rho", "lambda")     
+    rownames(summary.hyper) <- c("tau2.S", "tau2.T", "tau2.I", "rho.S", "rho.T")     
     summary.hyper[1, 4:7] <- c(n.keep, 100, effectiveSize(mcmc(samples.tau2[ ,1])), geweke.diag(mcmc(samples.tau2[ ,1]))$z)     
     summary.hyper[2, 4:7] <- c(n.keep, 100, effectiveSize(mcmc(samples.tau2[ ,2])), geweke.diag(mcmc(samples.tau2[ ,2]))$z)   
     summary.hyper[3, 4:7] <- c(n.keep, 100, effectiveSize(mcmc(samples.tau2[ ,3])), geweke.diag(mcmc(samples.tau2[ ,3]))$z)   
-    summary.hyper[4, 4:7] <- c(n.keep, accept.rho, effectiveSize(mcmc(samples.rho)), geweke.diag(mcmc(samples.rho))$z)   
-    summary.hyper[5, 4:7] <- c(n.keep, accept.lambda, effectiveSize(mcmc(samples.lambda)), geweke.diag(mcmc(samples.lambda))$z)         
+
+    if(!fix.rho.S)
+    {
+        summary.hyper[4, 1:3] <- quantile(samples.rho, c(0.5, 0.025, 0.975))
+        summary.hyper[4, 4:7] <- c(n.keep, accept.rho, effectiveSize(samples.rho), geweke.diag(samples.rho)$z)
+    }else
+    {
+        summary.hyper[4, 1:3] <- c(rho, rho, rho)
+        summary.hyper[4, 4:7] <- rep(NA, 4)
+    }
+ 
+    if(!fix.rho.T)
+    {
+        summary.hyper[5, 1:3] <- quantile(samples.lambda, c(0.5, 0.025, 0.975))
+        summary.hyper[5, 4:7] <- c(n.keep, accept.lambda, effectiveSize(samples.lambda), geweke.diag(samples.lambda)$z)
+    }else
+    {
+        summary.hyper[5, 1:3] <- c(lambda, lambda, lambda)
+        summary.hyper[5, 4:7] <- rep(NA, 4)
+    }   
 }else
 {
     summary.hyper <- array(NA, c(4, 7))     
     summary.hyper[1,1:3] <- quantile(samples.tau2[ ,1], c(0.5, 0.025, 0.975))
     summary.hyper[2,1:3] <- quantile(samples.tau2[ ,2], c(0.5, 0.025, 0.975))
-    summary.hyper[3,1:3] <- quantile(samples.rho, c(0.5, 0.025, 0.975))
-    summary.hyper[4,1:3] <- quantile(samples.lambda, c(0.5, 0.025, 0.975))
-    rownames(summary.hyper) <- c("tau2.phi", "tau2.delta",  "rho.phi", "rho.delta")     
+    rownames(summary.hyper) <- c("tau2.S", "tau2.T",  "rho.S", "rho.T")     
     summary.hyper[1, 4:7] <- c(n.keep, 100, effectiveSize(mcmc(samples.tau2[ ,1])), geweke.diag(mcmc(samples.tau2[ ,1]))$z)     
     summary.hyper[2, 4:7] <- c(n.keep, 100, effectiveSize(mcmc(samples.tau2[ ,2])), geweke.diag(mcmc(samples.tau2[ ,2]))$z)   
-    summary.hyper[3, 4:7] <- c(n.keep, accept.rho, effectiveSize(mcmc(samples.rho)), geweke.diag(mcmc(samples.rho))$z)   
-    summary.hyper[4, 4:7] <- c(n.keep, accept.lambda, effectiveSize(mcmc(samples.lambda)), geweke.diag(mcmc(samples.lambda))$z)     
+    
+    if(!fix.rho.S)
+    {
+        summary.hyper[3, 1:3] <- quantile(samples.rho, c(0.5, 0.025, 0.975))
+        summary.hyper[3, 4:7] <- c(n.keep, accept.rho, effectiveSize(samples.rho), geweke.diag(samples.rho)$z)
+    }else
+    {
+        summary.hyper[3, 1:3] <- c(rho, rho, rho)
+        summary.hyper[3, 4:7] <- rep(NA, 4)
+    }
+    
+    if(!fix.rho.T)
+    {
+        summary.hyper[4, 1:3] <- quantile(samples.lambda, c(0.5, 0.025, 0.975))
+        summary.hyper[4, 4:7] <- c(n.keep, accept.lambda, effectiveSize(samples.lambda), geweke.diag(samples.lambda)$z)
+    }else
+    {
+        summary.hyper[4, 1:3] <- c(lambda, lambda, lambda)
+        summary.hyper[4, 4:7] <- rep(NA, 4)
+    }   
 }
 summary.results <- rbind(summary.beta, summary.hyper)
 summary.results[ , 1:3] <- round(summary.results[ , 1:3], 4)
@@ -783,19 +923,42 @@ summary.results[ , 4:7] <- round(summary.results[ , 4:7], 1)
 
     
 ## Compile and return the results
-modelfit <- c(DIC, p.d, LMPL)
-names(modelfit) <- c("DIC", "p.d", "LMPL")
-samples.rhoext <- cbind(samples.rho, samples.lambda)
+modelfit <- c(DIC, p.d, WAIC, p.w, LMPL)
+names(modelfit) <- c("DIC", "p.d", "WAIC", "p.w", "LMPL")
+
+if(fix.rho.S & fix.rho.T)
+{
+    samples.rhoext <- NA
+}else if(fix.rho.S & !fix.rho.T)
+{
+    samples.rhoext <- samples.lambda
+    names(samples.rhoext) <- "rho.T"
+}else if(!fix.rho.S & fix.rho.T)
+{
+    samples.rhoext <- samples.rho  
+    names(samples.rhoext) <- "rho.S"
+}else
+{
+    samples.rhoext <- cbind(samples.rho, samples.lambda)
+    colnames(samples.rhoext) <- c("rho.S", "rho.T")
+}
+
+
+if(n.miss==0) samples.Y = NA
+if(!interaction) samples.gamma = NA
+
 
     if(interaction)
     {
-        samples <- list(beta=mcmc(samples.beta.orig), phi=mcmc(samples.phi),  delta=mcmc(samples.delta), gamma=mcmc(samples.gamma), tau2=mcmc(samples.tau2), rho=mcmc(samples.rhoext), fitted=mcmc(samples.fitted))        
         model.string <- c("Likelihood model - binomial (logit link function)", "\nLatent structure model - spatial and temporal main effects and an interaction\n")
+        colnames(samples.tau2) <- c("tau2.S", "tau2.T", "tau2.I")
     }else
     {
-        samples <- list(beta=mcmc(samples.beta.orig), phi=mcmc(samples.phi),  delta=mcmc(samples.delta), tau2=mcmc(samples.tau2), rho=mcmc(samples.rhoext), fitted=mcmc(samples.fitted))        
         model.string <- c("Likelihood model - binomial (logit link function)", "\nLatent structure model - spatial and temporal main effects\n")
+        colnames(samples.tau2) <- c("tau2.S", "tau2.T")
     }
+
+    samples <- list(beta=mcmc(samples.beta.orig), phi=mcmc(samples.phi),  delta=mcmc(samples.delta), gamma=mcmc(samples.gamma), tau2=mcmc(samples.tau2), rho=mcmc(samples.rhoext), fitted=mcmc(samples.fitted), Y=mcmc(samples.Y))        
     results <- list(summary.results=summary.results, samples=samples, fitted.values=fitted.values, residuals=residuals, modelfit=modelfit, accept=accept.final, localised.structure=NULL, formula=formula, model=model.string,  X=X)
     class(results) <- "carbayesST"
     if(verbose)
