@@ -1,4 +1,4 @@
-binomial.CARlinear <- function(formula, data=NULL, trials, W, burnin, n.sample, thin=1,  prior.mean.beta=NULL, prior.var.beta=NULL, prior.mean.alpha=NULL, prior.var.alpha=NULL, prior.tau2=NULL, verbose=TRUE)
+binomial.CARlinear <- function(formula, data=NULL, trials, W, burnin, n.sample, thin=1,  prior.mean.beta=NULL, prior.var.beta=NULL, prior.mean.alpha=NULL, prior.var.alpha=NULL, prior.tau2=NULL, fix.rho.slo=FALSE, rho.slo=NULL, fix.rho.int=FALSE, rho.int=NULL,  verbose=TRUE)
 {
     #### Check on the verbose option
     if(is.null(verbose)) verbose=TRUE     
@@ -67,26 +67,49 @@ X.indicator <- rep(NA, p)       # To determine which parameter estimates to tran
     
     
     
+#### Format and check the neighbourhood matrix W
+if(!is.matrix(W)) stop("W is not a matrix.", call.=FALSE)
+K <- nrow(W)
+if(ncol(W)!= K) stop("W has the wrong number of columns.", call.=FALSE)
+if(nrow(W)!= K) stop("W has the wrong number of rows.", call.=FALSE)
+if(floor(N.all/K)!=ceiling(N.all/K)) stop("The number of spatial areas is not a multiple of the number of data points.", call.=FALSE)
+N <- N.all / K
+if(sum(is.na(W))>0) stop("W has missing 'NA' values.", call.=FALSE)
+if(!is.numeric(W)) stop("W has non-numeric values.", call.=FALSE)
+if(min(W)<0) stop("W has negative elements.", call.=FALSE)
+if(sum(W!=t(W))>0) stop("W is not symmetric.", call.=FALSE)
+if(min(apply(W, 1, sum))==0) stop("W has some areas with no neighbours (one of the row sums equals zero).", call.=FALSE)    
+
+
 #### Response variable
 Y <- model.response(frame)
-    
-## Check for errors
-    if(sum(is.na(trials))>0) stop("the numbers of trials has missing 'NA' values.", call.=FALSE)
-    if(!is.numeric(trials)) stop("the numbers of trials has non-numeric values.", call.=FALSE)
-    int.check <- N.all-sum(ceiling(trials)==floor(trials))
-    if(int.check > 0) stop("the numbers of trials has non-integer values.", call.=FALSE)
-    if(min(trials)<=0) stop("the numbers of trials has zero or negative values.", call.=FALSE)
-    
-    if(sum(is.na(Y))>0) stop("the response has missing 'NA' values.", call.=FALSE)
-    if(!is.numeric(Y)) stop("the response variable has non-numeric values.", call.=FALSE)
-int.check <- N.all-sum(ceiling(Y)==floor(Y))
-    if(int.check > 0) stop("the respons variable has non-integer values.", call.=FALSE)
-    if(min(Y)<0) stop("the response variable has negative values.", call.=FALSE)
-    if(sum(Y>trials)>0) stop("the response variable has larger values that the numbers of trials.", call.=FALSE)
 Y <- as.numeric(Y)
 failures <- trials - Y
-    
-    
+
+## Check for errors
+if(sum(is.na(trials))>0) stop("the numbers of trials has missing 'NA' values.", call.=FALSE)
+if(!is.numeric(trials)) stop("the numbers of trials has non-numeric values.", call.=FALSE)
+int.check <- N.all-sum(ceiling(trials)==floor(trials))
+if(int.check > 0) stop("the numbers of trials has non-integer values.", call.=FALSE)
+if(min(trials)<=0) stop("the numbers of trials has zero or negative values.", call.=FALSE)
+
+## Create the missing value indicator
+which.miss <- as.numeric(!is.na(Y))
+which.miss.mat <- matrix(which.miss, nrow=K, ncol=N, byrow=FALSE)
+n.miss <- N.all - sum(which.miss)
+Y.miss <- Y
+Y.miss[which.miss==0] <- median(Y, na.rm=TRUE)
+failures.miss <- failures
+failures.miss[which.miss==0] <- median(failures, na.rm=TRUE)
+
+if(!is.numeric(Y)) stop("the response variable has non-numeric values.", call.=FALSE)
+int.check <- N.all- n.miss - sum(ceiling(Y)==floor(Y), na.rm=TRUE)
+if(int.check > 0) stop("the response variable has non-integer values.", call.=FALSE)
+if(min(Y, na.rm=TRUE)<0) stop("the response variable has negative values.", call.=FALSE)
+if(sum(Y>trials, na.rm=TRUE)>0) stop("the response variable has larger values that the numbers of trials.", call.=FALSE)
+
+
+
 #### Offset variable
 offset <- try(model.offset(frame), silent=TRUE)
     if(class(offset)=="try-error")   stop("the offset is not numeric.", call.=FALSE)
@@ -96,39 +119,48 @@ offset <- try(model.offset(frame), silent=TRUE)
     
     
     
-#### Format and check the neighbourhood matrix W
-    if(!is.matrix(W)) stop("W is not a matrix.", call.=FALSE)
-K <- nrow(W)
-    if(ncol(W)!= K) stop("W has the wrong number of columns.", call.=FALSE)
-    if(nrow(W)!= K) stop("W has the wrong number of rows.", call.=FALSE)
-    if(floor(N.all/K)!=ceiling(N.all/K)) stop("The number of spatial areas is not a multiple of the number of data points.", call.=FALSE)
-N <- N.all / K
-    if(sum(is.na(W))>0) stop("W has missing 'NA' values.", call.=FALSE)
-    if(!is.numeric(W)) stop("W has non-numeric values.", call.=FALSE)
-    if(min(W)<0) stop("W has negative elements.", call.=FALSE)
-    if(sum(W!=t(W))>0) stop("W is not symmetric.", call.=FALSE)
-    if(min(apply(W, 1, sum))==0) stop("W has some areas with no neighbours (one of the row sums equals zero).", call.=FALSE)    
+
 
         
 #### Specify the initial parameter values
 time <-(1:N - mean(1:N))/N
 time.all <- kronecker(time, rep(1,K))
 dat <- cbind(Y, failures)
-beta <- glm(dat~X.standardised-1 + time.all, offset=offset, family=binomial)$coefficients[1:p]
-alpha <- glm(dat~X.standardised-1 + time.all, offset=offset, family=binomial)$coefficients[(p+1)]
+mod.glm <- glm(dat~X.standardised-1 + time.all, offset=offset, family="quasibinomial")
+beta.mean <- mod.glm$coefficients
+beta.sd <- sqrt(diag(summary(mod.glm)$cov.scaled))
+temp <- rnorm(n=length(beta.mean), mean=beta.mean, sd=beta.sd)
+beta <- temp[1:p]
+alpha <- temp[(p+1)]
+
 theta.hat <- Y / trials
 theta.hat[theta.hat==0] <- 0.01
 theta.hat[theta.hat==1] <- 0.99
-regression.vec <- as.numeric(X.standardised %*% beta) + time.all * alpha
-res.temp <- log(theta.hat / (1 - theta.hat)) - regression.vec - offset
-phi <- rnorm(n=K, mean=0, sd = 2*sd(res.temp))
-delta <- rnorm(n=K, mean=0, sd = 2*sd(res.temp))
-tau2.phi <- runif(1, min=var(res.temp)/2, max=var(res.temp)*2)
-tau2.delta <- runif(1, min=var(res.temp)/2, max=var(res.temp)*2)
-rho <- runif(1)  
-lambda <- runif(1)  
-    
-    
+res.temp <- log(theta.hat / (1 - theta.hat)) - as.numeric(X.standardised %*% beta) - time.all * alpha - offset
+res.sd <- sd(res.temp, na.rm=TRUE)/5
+phi <- rnorm(n=K, mean=0, sd = res.sd)
+delta <- rnorm(n=K, mean=0, sd = res.sd)
+tau2.phi <- var(phi)/10
+tau2.delta <- var(delta)/10
+
+if(fix.rho.int)
+{
+    rho <- rho.int
+}else
+{
+    rho <- runif(1)       
+}
+
+if(fix.rho.slo)
+{
+    lambda <- rho.slo
+}else
+{
+    lambda <- runif(1)       
+}   
+
+
+
     
 #### Check and specify the priors
     if(is.null(prior.mean.beta)) prior.mean.beta <- rep(0, p)
@@ -204,18 +236,37 @@ lambda <- runif(1)
     
     
     
+    ## Check for errors on rho and fix.rho
+    if(!is.logical(fix.rho.int)) stop("fix.rho.int is not logical.", call.=FALSE)   
+    if(fix.rho.int & is.null(rho.int)) stop("rho.int is fixed but an initial value was not set.", call.=FALSE)   
+    if(fix.rho.int & !is.numeric(rho.int) ) stop("rho.S is not numeric.", call.=FALSE)  
+    if(rho<0 ) stop("rho.int is outside the range [0, 1].", call.=FALSE)  
+    if(rho>1 ) stop("rho.int is outside the range [0, 1].", call.=FALSE)  
+    
+    ## Check for errors on rho and fix.rho
+    if(!is.logical(fix.rho.slo)) stop("fix.rho.slo is not logical.", call.=FALSE)   
+    if(fix.rho.slo & is.null(rho.slo)) stop("rho.slo is fixed but an initial value was not set.", call.=FALSE)   
+    if(fix.rho.slo & !is.numeric(rho.slo) ) stop("rho.slo is not numeric.", call.=FALSE)  
+    if(lambda<0 ) stop("rho.slo is outside the range [0, 1].", call.=FALSE)  
+    if(lambda>1 ) stop("rho.slo is outside the range [0, 1].", call.=FALSE)  
+    
+
+        
 #### Set up matrices to store samples
 n.keep <- floor((n.sample - burnin)/thin)
 samples.beta <- array(NA, c(n.keep, p))
 samples.alpha <- array(NA, c(n.keep, 1))
 samples.phi <- array(NA, c(n.keep, K))
 samples.delta <- array(NA, c(n.keep, K))
-samples.rho <- array(NA, c(n.keep, 1))
-samples.lambda <- array(NA, c(n.keep, 1))
+if(!fix.rho.int) samples.rho <- array(NA, c(n.keep, 1))
+if(!fix.rho.slo) samples.lambda <- array(NA, c(n.keep, 1))
 samples.tau2 <- array(NA, c(n.keep, 2))
-colnames(samples.tau2) <- c("tau2.phi", "tau2.delta")
+colnames(samples.tau2) <- c("tau2.int", "tau2.slo")
 samples.fitted <- array(NA, c(n.keep, N.all))
+samples.like <- array(NA, c(n.keep, N.all))
 samples.deviance <- array(NA, c(n.keep, 1))
+if(n.miss>0) samples.Y <- array(NA, c(n.keep, n.miss))
+
     
     
     
@@ -268,27 +319,39 @@ temp <- 1
 Wstar <- diag(apply(W,1,sum)) - W
 Wstar.eigen <- eigen(Wstar)
 Wstar.val <- Wstar.eigen$values
-det.Q.rho <-  0.5 * sum(log((rho * Wstar.val + (1-rho))))    
-det.Q.lambda <-  0.5 * sum(log((lambda * Wstar.val + (1-lambda))))      
+if(!fix.rho.int) det.Q.rho <-  0.5 * sum(log((rho * Wstar.val + (1-rho))))    
+if(!fix.rho.slo) det.Q.lambda <-  0.5 * sum(log((lambda * Wstar.val + (1-lambda))))     
     
     
 #### Specify quantities that do not change
 offset.mat <- matrix(offset, nrow=K, ncol=N, byrow=FALSE) 
 regression.mat <- matrix(X.standardised %*% beta, nrow=K, ncol=N, byrow=FALSE)   
 Y.mat <- matrix(Y, nrow=K, ncol=N, byrow=FALSE)
+Y.mat.miss <- matrix(Y.miss, nrow=K, ncol=N, byrow=FALSE)
 failures.mat <- matrix(failures, nrow=K, ncol=N, byrow=FALSE)
+failures.mat.miss <- matrix(failures.miss, nrow=K, ncol=N, byrow=FALSE)
 phi.mat <- matrix(rep(phi, N), byrow=F, nrow=K)
 time.mat <- matrix(rep(time, K), byrow=TRUE, nrow=K)    
 delta.time.mat <- apply(time.mat, 2, "*", delta)
     
     
+## Check for islands
+W.list<- mat2listw(W)
+W.nb <- W.list$neighbours
+W.islands <- n.comp.nb(W.nb)
+islands <- W.islands$comp.id
+n.islands <- max(W.islands$nc)
+if(rho==1) tau2.phi.shape <- prior.tau2[1] + 0.5 * (K-n.islands)   
+if(lambda==1) tau2.delta.shape <- prior.tau2[1] + 0.5 * (K-n.islands)     
+
+
 ###########################
 #### Run the Bayesian model
 ###########################
 ## Start timer
     if(verbose)
     {
-        cat("Collecting", n.sample, "samples\n", sep = " ")
+        cat("Generating", n.sample, "samples\n", sep = " ")
         progressBar <- txtProgressBar(style = 3)
         percentage.points<-round((1:100/100)*n.sample)
     }else
@@ -309,7 +372,7 @@ delta.time.mat <- apply(time.mat, 2, "*", delta)
         for(r in 1:n.beta.block)
         {
             proposal.beta[beta.beg[r]:beta.fin[r]] <- proposal[beta.beg[r]:beta.fin[r]]
-            prob <- binomialbetaupdate(X.standardised, N.all, p, beta, proposal.beta, offset.temp, Y, failures, prior.mean.beta, prior.var.beta)
+            prob <- binomialbetaupdate(X.standardised, N.all, p, beta, proposal.beta, offset.temp, Y.miss, failures.miss, prior.mean.beta, prior.var.beta, which.miss)
             if(prob > runif(1))
             {
                 beta[beta.beg[r]:beta.fin[r]] <- proposal.beta[beta.beg[r]:beta.fin[r]]
@@ -336,7 +399,7 @@ delta.time.mat <- apply(time.mat, 2, "*", delta)
         p.proposal <- exp(lp.proposal) / (1 + exp(lp.proposal))
         like.current <- Y * log(p.current) + failures * log(1-p.current)
         like.proposal <- Y * log(p.proposal) + failures * log(1-p.proposal)
-        prob2 <- sum(like.proposal - like.current)
+        prob2 <- sum(like.proposal - like.current, na.rm=TRUE)
         prob <- exp(prob1 + prob2)
             if(prob > runif(1))
             {
@@ -354,9 +417,15 @@ delta.time.mat <- apply(time.mat, 2, "*", delta)
         ## Sample from phi
         ####################
         phi.offset <- offset.mat + regression.mat + delta.time.mat + alpha * time.mat
-        temp1 <- binomialcarupdate(W.triplet, W.begfin, W.triplet.sum, K, phi, tau2.phi,Y.mat, failures.mat, proposal.sd.phi, rho, phi.offset, N, rep(1,N))
+        temp1 <- binomialcarupdate(W.triplet, W.begfin, W.triplet.sum, K, phi, tau2.phi,Y.mat.miss, failures.mat.miss, proposal.sd.phi, rho, phi.offset, N, rep(1,N), which.miss.mat)
         phi <- temp1[[1]]
-        phi <- phi - mean(phi)
+        if(rho<1)
+        {
+            phi <- phi - mean(phi)
+        }else
+        {
+            phi[which(islands==1)] <- phi[which(islands==1)] - mean(phi[which(islands==1)])   
+        }
         phi.mat <- matrix(rep(phi, N), byrow=F, nrow=K)    
         accept[5] <- accept[5] + temp1[[2]]
         accept[6] <- accept[6] + K  
@@ -367,9 +436,15 @@ delta.time.mat <- apply(time.mat, 2, "*", delta)
         ## Sample from delta
         ####################
         delta.offset <- offset.mat + regression.mat + phi.mat +  alpha * time.mat
-        temp2 <- binomialcarupdate(W.triplet, W.begfin, W.triplet.sum, K, delta, tau2.delta,Y.mat, failures.mat, proposal.sd.delta, lambda, delta.offset, N, time)
+        temp2 <- binomialcarupdate(W.triplet, W.begfin, W.triplet.sum, K, delta, tau2.delta,Y.mat.miss, failures.mat.miss, proposal.sd.delta, lambda, delta.offset, N, time, which.miss.mat)
         delta <- temp2[[1]]
-        delta <- delta - mean(delta)
+        if(lambda <1)
+        {
+            delta <- delta - mean(delta)
+        }else
+        {
+            delta[which(islands==1)] <- delta[which(islands==1)] - mean(delta[which(islands==1)])   
+        }
         delta.time.mat <- apply(time.mat, 2, "*", delta)
         accept[7] <- accept[7] + temp2[[2]]
         accept[8] <- accept[8] + K      
@@ -397,6 +472,8 @@ delta.time.mat <- apply(time.mat, 2, "*", delta)
         ##################
         ## Sample from rho
         ##################
+        if(!fix.rho.int)
+        {
         proposal.rho <- rtrunc(n=1, spec="norm", a=0, b=1, mean=rho, sd=proposal.sd.rho)   
         temp3 <- quadform(W.triplet, W.triplet.sum, W.n.triplet, K, phi, phi, proposal.rho)
         det.Q.proposal <- 0.5 * sum(log((proposal.rho * Wstar.val + (1-proposal.rho))))              
@@ -414,12 +491,15 @@ delta.time.mat <- apply(time.mat, 2, "*", delta)
         {
         }              
         accept[10] <- accept[10] + 1           
-        
+        }else
+        {}
         
         
         #####################
         ## Sample from lambda
         #####################
+        if(!fix.rho.slo)
+        {
         proposal.lambda <- rtrunc(n=1, spec="norm", a=0, b=1, mean=lambda, sd=proposal.sd.lambda)   
         temp3 <- quadform(W.triplet, W.triplet.sum, W.n.triplet, K, delta, delta, proposal.lambda)
         det.Q.proposal <- 0.5 * sum(log((proposal.lambda * Wstar.val + (1-proposal.lambda))))              
@@ -437,6 +517,8 @@ delta.time.mat <- apply(time.mat, 2, "*", delta)
         {
         }              
         accept[12] <- accept[12] + 1           
+        }else
+        {}
         
         
         
@@ -447,7 +529,8 @@ delta.time.mat <- apply(time.mat, 2, "*", delta)
         prob <- exp(lp) / (1+exp(lp))
         fitted <- trials * prob
         deviance.all <- dbinom(x=Y, size=trials, prob=prob, log=TRUE)
-        deviance <- -2 * sum(deviance.all)     
+        like <- exp(deviance.all)
+        deviance <- -2 * sum(deviance.all, na.rm=TRUE)   
         
         
         
@@ -461,11 +544,14 @@ delta.time.mat <- apply(time.mat, 2, "*", delta)
             samples.phi[ele, ] <- phi
             samples.delta[ele, ] <- delta
             samples.alpha[ele, ] <- alpha
-            samples.rho[ele, ] <- rho
-            samples.lambda[ele, ] <- lambda
+            if(!fix.rho.int) samples.rho[ele, ] <- rho
+            if(!fix.rho.slo) samples.lambda[ele, ] <- lambda
             samples.tau2[ele, ] <- c(tau2.phi, tau2.delta)
             samples.deviance[ele, ] <- deviance
             samples.fitted[ele, ] <- fitted
+            samples.like[ele, ] <- like
+            if(n.miss>0) samples.Y[ele, ] <- rbinom(n=n.miss, size=trials[which.miss==0], prob=prob[which.miss==0])
+            
         }else
         {
         }
@@ -483,70 +569,72 @@ delta.time.mat <- apply(time.mat, 2, "*", delta)
             accept.phi <- 100 * accept[5] / accept[6]
             accept.delta <- 100 * accept[7] / accept[8]
             accept.rho <- 100 * accept[9] / accept[10]
+            if(is.na(accept.rho)) accept.rho <- 45
             accept.lambda <- 100 * accept[11] / accept[12]
+            if(is.na(accept.lambda)) accept.lambda <- 45
             accept.all <- accept.all + accept
             accept <- rep(0,12)
             
             #### beta tuning parameter
             if(accept.beta > 40)
             {
-                proposal.sd.beta <- 2 * proposal.sd.beta
+                proposal.sd.beta <- proposal.sd.beta + 0.1 * proposal.sd.beta
             }else if(accept.beta < 20)              
             {
-                proposal.sd.beta <- 0.5 * proposal.sd.beta
+                proposal.sd.beta <- proposal.sd.beta - 0.1 * proposal.sd.beta
             }else
             {
             }
             #### alpha tuning parameter
             if(accept.alpha > 50)
             {
-                proposal.sd.alpha <- 2 * proposal.sd.alpha
+                proposal.sd.alpha <- proposal.sd.alpha + 0.1 * proposal.sd.alpha
             }else if(accept.alpha < 40)              
             {
-                proposal.sd.alpha <- 0.5 * proposal.sd.alpha
+                proposal.sd.alpha <- proposal.sd.alpha - 0.1 * proposal.sd.alpha
             }else
             {
             }      
             #### phi tuning parameter
             if(accept.phi > 50)
             {
-                proposal.sd.phi <- 2 * proposal.sd.phi
+                proposal.sd.phi <- proposal.sd.phi + 0.1 * proposal.sd.phi
             }else if(accept.phi < 40)              
             {
-                proposal.sd.phi <- 0.5 * proposal.sd.phi
+                proposal.sd.phi <- proposal.sd.phi - 0.1 * proposal.sd.phi
             }else
             {
-            }
+            }      
             #### delta tuning parameter
             if(accept.delta > 50)
             {
-                proposal.sd.delta <- 2 * proposal.sd.delta
+                proposal.sd.delta <- proposal.sd.delta + 0.1 * proposal.sd.delta
             }else if(accept.delta < 40)              
             {
-                proposal.sd.delta <- 0.5 * proposal.sd.delta
+                proposal.sd.delta <- proposal.sd.delta - 0.1 * proposal.sd.delta
             }else
             {
             }
             #### rho tuning parameter
             if(accept.rho > 50)
             {
-                proposal.sd.rho <- min(2 * proposal.sd.rho, 0.5)
+                proposal.sd.rho <- min(proposal.sd.rho + 0.1 * proposal.sd.rho, 0.5)
             }else if(accept.rho < 40)              
             {
-                proposal.sd.rho <- 0.5 * proposal.sd.rho
+                proposal.sd.rho <- proposal.sd.rho - 0.1 * proposal.sd.rho
             }else
             {
             }
             #### lambda tuning parameter
             if(accept.lambda > 50)
             {
-                proposal.sd.lambda <- min(2 * proposal.sd.lambda, 0.5)
+                proposal.sd.lambda <- min(proposal.sd.lambda + 0.1 * proposal.sd.lambda, 0.5)
             }else if(accept.lambda < 40)              
             {
-                proposal.sd.lambda <- 0.5 * proposal.sd.lambda
+                proposal.sd.lambda <- proposal.sd.lambda - 0.1 * proposal.sd.lambda
             }else
             {
-            }      
+            }    
         }else
         {   
         }
@@ -578,10 +666,26 @@ accept.beta <- 100 * accept.all[1] / accept.all[2]
 accept.alpha <- 100 * accept.all[3] / accept.all[4]
 accept.phi <- 100 * accept.all[5] / accept.all[6]
 accept.delta <- 100 * accept.all[7] / accept.all[8]
-accept.rho <- 100 * accept.all[9] / accept.all[10]
-accept.lambda <- 100 * accept.all[11] / accept.all[12]
+
+if(!fix.rho.int)
+{
+    accept.rho <- 100 * accept.all[9] / accept.all[10]
+}else
+{
+    accept.rho <- NA    
+}
+
+if(!fix.rho.slo)
+{
+    accept.lambda <- 100 * accept.all[11] / accept.all[12]
+}else
+{
+    accept.lambda <- NA    
+}
+
 accept.final <- c(accept.beta, accept.alpha, accept.phi, accept.delta, accept.rho, accept.lambda)
-names(accept.final) <- c("beta", "alpha", "phi", "delta", "rho", "lambda")
+names(accept.final) <- c("beta", "alpha", "phi", "delta", "rho.int", "rho.slo")
+
     
     
 ## Compute DIC
@@ -595,19 +699,28 @@ regression.mat <- matrix(X.standardised %*% median.beta, nrow=K, ncol=N, byrow=F
 lp.median <- offset.mat + regression.mat + phi.mat + delta.time.mat + median.alpha * time.mat
 median.prob <- exp(lp.median)  / (1 + exp(lp.median))
 fitted.median <- trials * median.prob
-deviance.fitted <- -2 * sum(dbinom(x=Y, size=trials, prob=median.prob, log=TRUE))
+deviance.fitted <- -2 * sum(dbinom(x=Y, size=trials, prob=median.prob, log=TRUE), na.rm=TRUE)
 p.d <- median(samples.deviance) - deviance.fitted
 DIC <- 2 * median(samples.deviance) - deviance.fitted  
     
+
+
+#### Watanabe-Akaike Information Criterion (WAIC)
+LPPD <- sum(log(apply(samples.like,2,mean)), na.rm=TRUE)
+p.w <- sum(apply(log(samples.like),2,var), na.rm=TRUE)
+WAIC <- -2 * (LPPD - p.w)
+
+
 ## Compute the LMPL
 CPO <- rep(NA, N.all)
     for(j in 1:N.all)
     {
     CPO[j] <- 1/median((1 / dbinom(x=Y[j], size=trials[j], prob=(samples.fitted[ ,j] / trials[j]))))    
     }
-LMPL <- sum(log(CPO))   
-    
+LMPL <- sum(log(CPO), na.rm=TRUE)   
     
+   
+ 
 ## Create the Fitted values
 fitted.values <- apply(samples.fitted, 2, median)
 residuals <- as.numeric(Y) - fitted.values
@@ -653,24 +766,60 @@ summary.hyper <- array(NA, c(5, 7))
 summary.hyper[1,1:3] <- quantile(samples.alpha, c(0.5, 0.025, 0.975))
 summary.hyper[2,1:3] <- quantile(samples.tau2[ ,1], c(0.5, 0.025, 0.975))
 summary.hyper[3,1:3] <- quantile(samples.tau2[ ,2], c(0.5, 0.025, 0.975))
-summary.hyper[4,1:3] <- quantile(samples.rho, c(0.5, 0.025, 0.975))
-summary.hyper[5,1:3] <- quantile(samples.lambda, c(0.5, 0.025, 0.975))
-rownames(summary.hyper) <- c("alpha", "tau2.phi", "tau2.delta",  "rho.phi", "rho.delta")     
+rownames(summary.hyper) <- c("alpha", "tau2.int", "tau2.slo",  "rho.int", "rho.slo")     
 summary.hyper[1, 4:7] <- c(n.keep, accept.alpha, effectiveSize(mcmc(samples.alpha)), geweke.diag(mcmc(samples.alpha))$z)     
 summary.hyper[2, 4:7] <- c(n.keep, 100, effectiveSize(mcmc(samples.tau2[ ,1])), geweke.diag(mcmc(samples.tau2[ ,1]))$z)   
 summary.hyper[3, 4:7] <- c(n.keep, 100, effectiveSize(mcmc(samples.tau2[ ,2])), geweke.diag(mcmc(samples.tau2[ ,2]))$z)   
-summary.hyper[4, 4:7] <- c(n.keep, accept.rho, effectiveSize(mcmc(samples.rho)), geweke.diag(mcmc(samples.rho))$z)   
-summary.hyper[5, 4:7] <- c(n.keep, accept.lambda, effectiveSize(mcmc(samples.lambda)), geweke.diag(mcmc(samples.lambda))$z)         
+
+if(!fix.rho.int)
+{
+    summary.hyper[4, 1:3] <- quantile(samples.rho, c(0.5, 0.025, 0.975))
+    summary.hyper[4, 4:7] <- c(n.keep, accept.rho, effectiveSize(samples.rho), geweke.diag(samples.rho)$z)
+}else
+{
+    summary.hyper[4, 1:3] <- c(rho, rho, rho)
+    summary.hyper[4, 4:7] <- rep(NA, 4)
+}
+
+if(!fix.rho.slo)
+{
+    summary.hyper[5, 1:3] <- quantile(samples.lambda, c(0.5, 0.025, 0.975))
+    summary.hyper[5, 4:7] <- c(n.keep, accept.lambda, effectiveSize(samples.lambda), geweke.diag(samples.lambda)$z)
+}else
+{
+    summary.hyper[5, 1:3] <- c(lambda, lambda, lambda)
+    summary.hyper[5, 4:7] <- rep(NA, 4)
+}   
 
 summary.results <- rbind(summary.beta, summary.hyper)
 summary.results[ , 1:3] <- round(summary.results[ , 1:3], 4)
-summary.results[ , 4:7] <- round(summary.results[ , 4:7], 1)    
+summary.results[ , 4:7] <- round(summary.results[ , 4:7], 1)
+
     
 ## Compile and return the results
-modelfit <- c(DIC, p.d, LMPL)
-names(modelfit) <- c("DIC", "p.d", "LMPL")
-samples.rhoext <- cbind(samples.rho, samples.lambda)
-samples <- list(beta=mcmc(samples.beta.orig), alpha=mcmc(samples.alpha), phi=mcmc(samples.phi),  delta=mcmc(samples.delta), tau2=mcmc(samples.tau2), rho=mcmc(samples.rhoext), fitted=mcmc(samples.fitted))        
+modelfit <- c(DIC, p.d, WAIC, p.w, LMPL)
+names(modelfit) <- c("DIC", "p.d", "WAIC", "p.w", "LMPL")
+
+if(fix.rho.int & fix.rho.slo)
+{
+    samples.rhoext <- NA
+}else if(fix.rho.int & !fix.rho.slo)
+{
+    samples.rhoext <- samples.lambda
+    names(samples.rhoext) <- "rho.slo"
+}else if(!fix.rho.int & fix.rho.slo)
+{
+    samples.rhoext <- samples.rho  
+    names(samples.rhoext) <- "rho.int"
+}else
+{
+    samples.rhoext <- cbind(samples.rho, samples.lambda)
+    colnames(samples.rhoext) <- c("rho.int", "rho.slo")
+}
+
+if(n.miss==0) samples.Y = NA
+
+samples <- list(beta=mcmc(samples.beta.orig), alpha=mcmc(samples.alpha), phi=mcmc(samples.phi),  delta=mcmc(samples.delta), tau2=mcmc(samples.tau2), rho=mcmc(samples.rhoext), fitted=mcmc(samples.fitted), Y=mcmc(samples.Y))        
 model.string <- c("Likelihood model - binomial (logit link function)", "\nLatent structure model - Spatially autocorrelated linear time trends\n")
 results <- list(summary.results=summary.results, samples=samples, fitted.values=fitted.values, residuals=residuals, modelfit=modelfit, accept=accept.final, localised.structure=NULL,  formula=formula, model=model.string, X=X)
 class(results) <- "carbayesST"
