@@ -23,7 +23,6 @@ binomial.CARlocalised <- function(formula, data=NULL, G, trials,  W, burnin, n.s
     if(!is.matrix(W)) stop("W is not a matrix.", call.=FALSE)
     K <- nrow(W)
     if(ncol(W)!= K) stop("W has the wrong number of columns.", call.=FALSE)
-    if(floor(N.all/K)!=ceiling(N.all/K)) stop("The number of spatial areas is not a multiple of the number of data points.", call.=FALSE)
     if(sum(is.na(W))>0) stop("W has missing 'NA' values.", call.=FALSE)
     if(!is.numeric(W)) stop("W has non-numeric values.", call.=FALSE)
     if(min(W)<0) stop("W has negative elements.", call.=FALSE)
@@ -34,6 +33,7 @@ binomial.CARlocalised <- function(formula, data=NULL, G, trials,  W, burnin, n.s
     Y <- model.response(frame)
     N.all <- length(Y)
     N <- N.all / K
+    if(floor(N.all/K)!=ceiling(N.all/K)) stop("The number of spatial areas is not a multiple of the number of data points.", call.=FALSE)
     if(sum(is.na(Y))>0) stop("the response has missing 'NA' values.", call.=FALSE)
     if(!is.numeric(Y)) stop("the response variable has non-numeric values.", call.=FALSE)
     int.check <- N.all-sum(ceiling(Y)==floor(Y))
@@ -42,7 +42,8 @@ binomial.CARlocalised <- function(formula, data=NULL, G, trials,  W, burnin, n.s
     failures <- trials - Y
     Y.mat <- matrix(Y, nrow=K, ncol=N, byrow=FALSE) 
     failures.mat <- matrix(failures, nrow=K, ncol=N, byrow=FALSE)  
-    
+    which.miss <- as.numeric(!is.na(Y))
+    which.miss.mat <- matrix(which.miss, nrow=K, ncol=N, byrow=FALSE)
      
     #### Offset variable
     ## Create the offset
@@ -103,10 +104,12 @@ binomial.CARlocalised <- function(formula, data=NULL, G, trials,  W, burnin, n.s
         
         ## Compute a starting value for beta
         dat <- cbind(Y, failures)
-        beta <- glm(dat~X.standardised-1, offset=offset, family=binomial)$coefficients
+        mod.glm <- glm(dat~X.standardised-1, offset=offset, family="quasibinomial")
+        beta.mean <- mod.glm$coefficients
+        beta.sd <- sqrt(diag(summary(mod.glm)$cov.scaled))
+        beta <- rnorm(n=length(beta.mean), mean=beta.mean, sd=beta.sd)
         regression.vec <- X.standardised %*% beta
         regression.mat <- matrix(regression.vec, nrow=K, ncol=N, byrow=FALSE)   
-        
     }
     
     
@@ -173,16 +176,18 @@ binomial.CARlocalised <- function(formula, data=NULL, G, trials,  W, burnin, n.s
     theta.hat[theta.hat==0] <- 0.01
     theta.hat[theta.hat==1] <- 0.99
     res.temp <- log(theta.hat / (1 - theta.hat)) - regression.vec - offset
+    res.sd <- sd(res.temp, na.rm=TRUE)/5
+    phi.mat <- matrix(rnorm(n=N.all, mean=0, sd = res.sd), nrow=K, byrow=FALSE)
+    tau2 <- var(phi)/10
+    gamma <- runif(1)
     Z <- sample(1:G, size=N.all, replace=TRUE)
-    lambda <- sort(runif(G, min=min(res.temp), max=max(res.temp)))
     Z.mat <- matrix(Z, nrow=K, ncol=N, byrow=FALSE)
+    lambda <- sort(runif(G, min=min(res.temp), max=max(res.temp)))
+    lambda.mat <- matrix(rep(lambda, N), nrow=N, byrow=TRUE)
     delta <- runif(1,1, min(2, prior.delta))
     mu <- matrix(lambda[Z], nrow=K, ncol=N, byrow=FALSE)
-    lambda.mat <- matrix(rep(lambda, N), nrow=N, byrow=TRUE)
-    phi.mat <- matrix(rnorm(n=N.all, mean=0, sd = sd(res.temp)/2), nrow=K, byrow=FALSE)
-    tau2 <- runif(1, min=0, max=var(res.temp))
-    gamma <- runif(1)
-    
+
+
     
     ## Compute the blocking structure for beta     
     if(!is.null(X))
@@ -223,6 +228,7 @@ binomial.CARlocalised <- function(formula, data=NULL, G, trials,  W, burnin, n.s
     samples.phi <- array(NA, c(n.keep, N.all))
     samples.fitted <- array(NA, c(n.keep, N.all))
     samples.deviance <- array(NA, c(n.keep, 1))
+    samples.like <- array(NA, c(n.keep, N.all))
     
     if(!is.null(X))
     {
@@ -283,7 +289,7 @@ binomial.CARlocalised <- function(formula, data=NULL, G, trials,  W, burnin, n.s
     ## Start timer
     if(verbose)
     {
-        cat("Collecting", n.sample, "samples\n", sep = " ")
+        cat("Generating", n.sample, "samples\n", sep = " ")
         progressBar <- txtProgressBar(style = 3)
         percentage.points<-round((1:100/100)*n.sample)
     }else
@@ -306,7 +312,7 @@ binomial.CARlocalised <- function(formula, data=NULL, G, trials,  W, burnin, n.s
             for(r in 1:n.beta.block)
             {
                 proposal.beta[beta.beg[r]:beta.fin[r]] <- proposal[beta.beg[r]:beta.fin[r]]
-                prob <- binomialbetaupdate(X.standardised, N.all, p, beta, proposal.beta, offset.temp, Y, failures, prior.mean.beta, prior.var.beta)
+                prob <- binomialbetaupdate(X.standardised, N.all, p, beta, proposal.beta, offset.temp, Y, failures, prior.mean.beta, prior.var.beta, which.miss)
                 if(prob > runif(1))
                 {
                     beta[beta.beg[r]:beta.fin[r]] <- proposal.beta[beta.beg[r]:beta.fin[r]]
@@ -399,7 +405,7 @@ binomial.CARlocalised <- function(formula, data=NULL, G, trials,  W, burnin, n.s
         #### Sample from phi
         ####################
         phi.offset <- mu + offset.mat + regression.mat
-        temp1 <- binomialarcarupdate(W.triplet, W.begfin, W.triplet.sum,  K, N, phi.mat, tau2, gamma, 1, Y.mat, failures.mat, proposal.sd.phi, phi.offset, W.triplet.sum)      
+        temp1 <- binomialarcarupdate(W.triplet, W.begfin, W.triplet.sum,  K, N, phi.mat, tau2, gamma, 1, Y.mat, failures.mat, proposal.sd.phi, phi.offset, W.triplet.sum, which.miss.mat)      
         phi.temp <- temp1[[1]]
         phi <- as.numeric(phi.temp)
             for(i in 1:G)
@@ -439,6 +445,7 @@ binomial.CARlocalised <- function(formula, data=NULL, G, trials,  W, burnin, n.s
         prob <- exp(lp) / (1+exp(lp))
         fitted <- trials * prob
         deviance.all <- dbinom(x=Y, size=trials, prob=prob, log=TRUE)
+        like <- exp(deviance.all)
         deviance <- -2 * sum(deviance.all) 
         
         
@@ -456,7 +463,7 @@ binomial.CARlocalised <- function(formula, data=NULL, G, trials,  W, burnin, n.s
             samples.gamma[ele, ] <- gamma
             samples.deviance[ele, ] <- deviance
             samples.fitted[ele, ] <- fitted
-            
+            samples.like[ele, ] <- like
             if(!is.null(X)) samples.beta[ele, ] <- beta        
         }else
         {
@@ -478,13 +485,13 @@ binomial.CARlocalised <- function(formula, data=NULL, G, trials,  W, burnin, n.s
                 accept.beta <- 100 * accept[7] / accept[8]
                 if(accept.beta > 40)
                 {
-                    proposal.sd.beta <- 2 * proposal.sd.beta
+                    proposal.sd.beta <- proposal.sd.beta + 0.1 * proposal.sd.beta
                 }else if(accept.beta < 20)              
                 {
-                    proposal.sd.beta <- 0.5 * proposal.sd.beta
+                    proposal.sd.beta <- proposal.sd.beta - 0.1 * proposal.sd.beta
                 }else
                 {
-                }    
+                }   
                 accept.all <- accept.all + accept
                 accept <- rep(0,8)
             }else
@@ -499,10 +506,10 @@ binomial.CARlocalised <- function(formula, data=NULL, G, trials,  W, burnin, n.s
             #### lambda tuning parameter
             if(accept.lambda > 40)
             {
-                proposal.sd.lambda <- min(2 * proposal.sd.lambda, 10)
+                proposal.sd.lambda <- min(proposal.sd.lambda + 0.1 * proposal.sd.lambda, 10)
             }else if(accept.lambda < 20)              
             {
-                proposal.sd.lambda <- 0.5 * proposal.sd.lambda
+                proposal.sd.lambda <- proposal.sd.lambda - 0.1 * proposal.sd.lambda
             }else
             {
             }
@@ -511,10 +518,10 @@ binomial.CARlocalised <- function(formula, data=NULL, G, trials,  W, burnin, n.s
             #### delta tuning parameter               
             if(accept.delta > 50)
             {
-                proposal.sd.delta <- min(2 * proposal.sd.delta, 10)
+                proposal.sd.delta <- min(proposal.sd.delta + 0.1 * proposal.sd.delta, 10)
             }else if(accept.delta < 40)              
             {
-                proposal.sd.delta <- 0.5 * proposal.sd.delta
+                proposal.sd.delta <- proposal.sd.delta - 0.1 * proposal.sd.delta
             }else
             {
             }         
@@ -523,10 +530,10 @@ binomial.CARlocalised <- function(formula, data=NULL, G, trials,  W, burnin, n.s
             #### phi tuning parameter
             if(accept.phi > 50)
             {
-                proposal.sd.phi <- 2 * proposal.sd.phi
+                proposal.sd.phi <- proposal.sd.phi + 0.1 * proposal.sd.phi
             }else if(accept.phi < 40)              
             {
-                proposal.sd.phi <- 0.5 * proposal.sd.phi
+                proposal.sd.phi <- proposal.sd.phi - 0.1 * proposal.sd.phi
             }else
             {
             }
@@ -561,16 +568,16 @@ binomial.CARlocalised <- function(formula, data=NULL, G, trials,  W, burnin, n.s
     accept.lambda <- 100 * accept.all[1] / accept.all[2]
     accept.delta <- 100 * accept.all[3] / accept.all[4]
     accept.phi <- 100 * accept.all[5] / accept.all[6]
-    
+    accept.gamma <- 100
     if(!is.null(X))
     {
         accept.beta <- 100 * accept.all[7] / accept.all[8]   
-        accept.final <- c(accept.beta, accept.lambda, accept.delta, accept.phi)
-        names(accept.final) <- c("beta", "lambda", "delta", "phi")   
+        accept.final <- c(accept.beta, accept.lambda, accept.delta, accept.phi, accept.gamma)
+        names(accept.final) <- c("beta", "lambda", "delta", "phi", "rho.T")   
     }else
     {
-        accept.final <- c(accept.lambda,  accept.delta, accept.phi)
-        names(accept.final) <- c("lambda", "delta", "phi")   
+        accept.final <- c(accept.lambda,  accept.delta, accept.phi, accept.gamma)
+        names(accept.final) <- c("lambda", "delta", "phi", "rho.T")   
     }
     
     
@@ -596,6 +603,13 @@ binomial.CARlocalised <- function(formula, data=NULL, G, trials,  W, burnin, n.s
     p.d <- median(samples.deviance) - deviance.fitted
     DIC <- 2 * median(samples.deviance) - deviance.fitted     
     
+    
+    #### Watanabe-Akaike Information Criterion (WAIC)
+    LPPD <- sum(log(apply(samples.like,2,mean)), na.rm=TRUE)
+    p.w <- sum(apply(log(samples.like),2,var), na.rm=TRUE)
+    WAIC <- -2 * (LPPD - p.w)
+    
+    
     ## Compute the LMPL
     CPO <- rep(NA, N.all)
     for(j in 1:N.all)
@@ -639,7 +653,7 @@ binomial.CARlocalised <- function(formula, data=NULL, G, trials,  W, burnin, n.s
     summary.hyper[1,1:3] <- quantile(samples.delta, c(0.5, 0.025, 0.975))
     summary.hyper[2,1:3] <- quantile(samples.tau2, c(0.5, 0.025, 0.975))
     summary.hyper[3,1:3] <- quantile(samples.gamma, c(0.5, 0.025, 0.975))
-    rownames(summary.hyper) <- c("delta", "tau2", "gamma")      
+    rownames(summary.hyper) <- c("delta", "tau2", "rho.T")      
     summary.hyper[1, 4:7] <- c(n.keep, accept.delta, effectiveSize(mcmc(samples.delta)), geweke.diag(mcmc(samples.delta))$z)   
     summary.hyper[2, 4:7] <- c(n.keep, 100, effectiveSize(mcmc(samples.tau2)), geweke.diag(mcmc(samples.tau2))$z)   
     summary.hyper[3, 4:7] <- c(n.keep, 100, effectiveSize(mcmc(samples.gamma)), geweke.diag(mcmc(samples.gamma))$z)   
@@ -650,13 +664,14 @@ binomial.CARlocalised <- function(formula, data=NULL, G, trials,  W, burnin, n.s
     summary.lambda <- matrix(summary.lambda, ncol=7)
     rownames(summary.lambda) <- paste("lambda", 1:G, sep="")
     
-     if(!is.null(X))
+    
+    if(!is.null(X))
     {
         samples.beta.orig <- mcmc(samples.beta.orig)
         summary.beta <- t(apply(samples.beta.orig, 2, quantile, c(0.5, 0.025, 0.975))) 
         summary.beta <- cbind(summary.beta, rep(n.keep, p), rep(accept.beta,p), effectiveSize(samples.beta.orig), geweke.diag(samples.beta.orig)$z)
         rownames(summary.beta) <- colnames(X)
-        colnames(summary.beta) <- c("Median", "2.5%", "97.5%", "n.sample", "% accept")
+        colnames(summary.beta) <- c("Median", "2.5%", "97.5%", "n.sample", "% accept", "n.effective", "Geweke.diag")
         summary.results <- rbind(summary.beta, summary.lambda, summary.hyper)    
     }else
     {
@@ -667,17 +682,14 @@ binomial.CARlocalised <- function(formula, data=NULL, G, trials,  W, burnin, n.s
     summary.results[ , 4:7] <- round(summary.results[ , 4:7], 1)
     colnames(summary.results) <- c("Median", "2.5%", "97.5%", "n.sample", "% accept", "n.effective", "Geweke.diag")    
     
-    ## Compile and return the results
-    modelfit <- c(DIC, p.d, LMPL)
-    names(modelfit) <- c("DIC", "p.d",  "LMPL")
-    if(!is.null(X))
-    {
-        samples <- list(beta=mcmc(samples.beta.orig), lambda=mcmc(samples.lambda),  Z=mcmc(samples.Z), delta=mcmc(samples.delta), phi = mcmc(samples.phi), tau2=mcmc(samples.tau2), gamma=mcmc(samples.gamma), fitted=mcmc(samples.fitted), deviance=mcmc(samples.deviance))
-    }else
-    {
-        samples <- list(lambda=mcmc(samples.lambda),  Z=mcmc(samples.Z), delta=mcmc(samples.delta), phi = mcmc(samples.phi), tau2=mcmc(samples.tau2), gamma=mcmc(samples.gamma), fitted=mcmc(samples.fitted), deviance=mcmc(samples.deviance))
-    }
     
+    
+    ## Compile and return the results
+    modelfit <- c(DIC, p.d, WAIC, p.w, LMPL)
+    names(modelfit) <- c("DIC", "p.d", "WAIC", "p.w", "LMPL")
+    if(is.null(X)) samples.beta.orig = NA
+
+    samples <- list(beta=mcmc(samples.beta.orig), lambda=mcmc(samples.lambda),  Z=mcmc(samples.Z), delta=mcmc(samples.delta), phi = mcmc(samples.phi), tau2=mcmc(samples.tau2), rho.T=mcmc(samples.gamma), fitted=mcmc(samples.fitted), deviance=mcmc(samples.deviance))
     model.string <- c("Likelihood model - Binomial (logit link function)", "\nLatent structure model - Localised autoregressive CAR model\n")
     results <- list(summary.results=summary.results, samples=samples, fitted.values=fitted.values, residuals=residuals, modelfit=modelfit, accept=accept.final, localised.structure=median.Z, formula=formula, model=model.string,  X=X)
     class(results) <- "carbayesST"
