@@ -1,4 +1,4 @@
-gaussian.CARar <- function(formula, data=NULL, W, burnin, n.sample, thin=1,  prior.mean.beta=NULL, prior.var.beta=NULL, prior.nu2=NULL, prior.tau2=NULL, verbose=TRUE)
+gaussian.CARar <- function(formula, data=NULL, W, burnin, n.sample, thin=1,  prior.mean.beta=NULL, prior.var.beta=NULL, prior.nu2=NULL, prior.tau2=NULL, fix.rho.S=FALSE, rho.S=NULL, fix.rho.T=FALSE, rho.T=NULL, verbose=TRUE)
 {
 #### Check on the verbose option
     if(is.null(verbose)) verbose=TRUE     
@@ -64,11 +64,29 @@ gaussian.CARar <- function(formula, data=NULL, W, burnin, n.sample, thin=1,  pri
     }
     
     
-#### Response variable
-Y <- model.response(frame)
-    if(sum(is.na(Y))>0) stop("the response has missing 'NA' values.", call.=FALSE)
+    #### Format and check the neighbourhood matrix W
+    if(!is.matrix(W)) stop("W is not a matrix.", call.=FALSE)
+    K <- nrow(W)
+    if(ncol(W)!= K) stop("W has the wrong number of columns.", call.=FALSE)
+    if(nrow(W)!= K) stop("W has the wrong number of rows.", call.=FALSE)
+    if(floor(N.all/K)!=ceiling(N.all/K)) stop("The number of spatial areas is not a multiple of the number of data points.", call.=FALSE)
+    N <- N.all / K
+    if(sum(is.na(W))>0) stop("W has missing 'NA' values.", call.=FALSE)
+    if(!is.numeric(W)) stop("W has non-numeric values.", call.=FALSE)
+    if(min(W)<0) stop("W has negative elements.", call.=FALSE)
+    if(sum(W!=t(W))>0) stop("W is not symmetric.", call.=FALSE)
+    if(min(apply(W, 1, sum))==0) stop("W has some areas with no neighbours (one of the row sums equals zero).", call.=FALSE)    
+    
+    #### Response variable
+    Y <- model.response(frame)
+    which.miss <- as.numeric(!is.na(Y))
+    which.miss.mat <- matrix(which.miss, nrow=K, ncol=N, byrow=FALSE)
+    n.miss <- N.all - sum(which.miss)
+    Y.miss <- Y
+    Y.miss[which.miss==0] <- median(Y, na.rm=TRUE)    
     if(!is.numeric(Y)) stop("the response variable has non-numeric values.", call.=FALSE)
-   
+    X.short <- X.standardised[which.miss==1, ]    
+    
     
 #### Offset variable
 offset <- try(model.offset(frame), silent=TRUE)
@@ -78,31 +96,36 @@ offset <- try(model.offset(frame), silent=TRUE)
     if(!is.numeric(offset)) stop("the offset variable has non-numeric values.", call.=FALSE)
     
     
-#### Format and check the neighbourhood matrix W
-    if(!is.matrix(W)) stop("W is not a matrix.", call.=FALSE)
-K <- nrow(W)
-    if(ncol(W)!= K) stop("W has the wrong number of columns.", call.=FALSE)
-    if(nrow(W)!= K) stop("W has the wrong number of rows.", call.=FALSE)
-    if(floor(N.all/K)!=ceiling(N.all/K)) stop("The number of spatial areas is not a multiple of the number of data points.", call.=FALSE)
-N <- N.all / K
-    if(sum(is.na(W))>0) stop("W has missing 'NA' values.", call.=FALSE)
-    if(!is.numeric(W)) stop("W has non-numeric values.", call.=FALSE)
-    if(min(W)<0) stop("W has negative elements.", call.=FALSE)
-    if(sum(W!=t(W))>0) stop("W is not symmetric.", call.=FALSE)
-    if(min(apply(W, 1, sum))==0) stop("W has some areas with no neighbours (one of the row sums equals zero).", call.=FALSE)    
+
 
     
 #### Specify the initial parameter values
-beta <- lm(Y~X.standardised-1, offset=offset)$coefficients
-regression.vec <- X.standardised %*% beta
-res.temp <- Y - regression.vec - offset
-phi <- rnorm(n=N.all, mean=0, sd = 2*sd(res.temp))
-tau2 <- runif(1, min=var(res.temp)/2, max=var(res.temp)*2)
-nu2 <- runif(1, min=var(res.temp)/2, max=var(res.temp)*2)
-rho <- runif(1)
-gamma <- runif(1)
+mod.glm <- glm(Y~X.standardised-1, offset=offset)
+beta.mean <- mod.glm$coefficients
+beta.sd <- sqrt(diag(summary(mod.glm)$cov.scaled))
+beta <- rnorm(n=length(beta.mean), mean=beta.mean, sd=beta.sd)
+
+res.temp <- Y - X.standardised %*% beta - offset
+res.sd <- sd(res.temp, na.rm=TRUE)/5
+phi <- rnorm(n=N.all, mean=0, sd = res.sd)
+tau2 <- var(phi)/10
+if(fix.rho.S)
+{
+    rho <- rho.S
+}else
+{
+    rho <- runif(1)       
+}
+
+if(fix.rho.T)
+{
+    gamma <- rho.T
+}else
+{
+    gamma <- runif(1)       
+}   
+
 
-    
 
 #### Check and specify the priors
 ## Put in default priors
@@ -145,17 +168,34 @@ gamma <- runif(1)
     if(thin!=round(thin)) stop("thin is not an integer.", call.=FALSE) 
     
         
-    
+## Check for errors on rho and fix.rho
+if(!is.logical(fix.rho.S)) stop("fix.rho.S is not logical.", call.=FALSE)   
+if(fix.rho.S & is.null(rho.S)) stop("rho.S is fixed but an initial value was not set.", call.=FALSE)   
+if(fix.rho.S & !is.numeric(rho.S) ) stop("rho.S is not numeric.", call.=FALSE)  
+if(rho<0 ) stop("rho.S is outside the range [0, 1].", call.=FALSE)  
+if(rho>1 ) stop("rho.S is outside the range [0, 1].", call.=FALSE)  
+
+## Check for errors on rho and fix.rho
+if(!is.logical(fix.rho.T)) stop("fix.rho.T is not logical.", call.=FALSE)   
+if(fix.rho.T & is.null(rho.T)) stop("rho.T is fixed but an initial value was not set.", call.=FALSE)   
+if(fix.rho.T & !is.numeric(rho.T) ) stop("rho.T is not numeric.", call.=FALSE)  
+if(gamma<0 ) stop("rho.T is outside the range [0, 1].", call.=FALSE)  
+if(gamma>1 ) stop("rho.T is outside the range [0, 1].", call.=FALSE)  
+
+
 #### Set up matrices to store samples
 n.keep <- floor((n.sample - burnin)/thin)
 samples.beta <- array(NA, c(n.keep, p))
 samples.phi <- array(NA, c(n.keep, N.all))
 samples.tau2 <- array(NA, c(n.keep, 1))
 samples.nu2 <- array(NA, c(n.keep, 1))
-samples.rho <- array(NA, c(n.keep, 1))
-samples.gamma <- array(NA, c(n.keep, 1))     
+if(!fix.rho.S) samples.rho <- array(NA, c(n.keep, 1))
+if(!fix.rho.T) samples.gamma <- array(NA, c(n.keep, 1))
 samples.fitted <- array(NA, c(n.keep, N.all))
+samples.like <- array(NA, c(n.keep, N.all))
 samples.deviance <- array(NA, c(n.keep, 1))
+if(n.miss>0) samples.Y <- array(NA, c(n.keep, n.miss))
+
     
     
 #### Specify the Metropolis quantities
@@ -166,7 +206,9 @@ tau2.shape <- prior.tau2[1] + N.all/2
 nu2.shape <- prior.nu2[1] + N.all/2        
     
     
-    #### Spatial quantities
+
+
+#### Spatial quantities
     ## Create the triplet object
     W.triplet <- c(NA, NA, NA)
     for(i in 1:K)
@@ -196,28 +238,55 @@ nu2.shape <- prior.nu2[1] + N.all/2
     
     
     ## Create the determinant     
-    Wstar <- diag(apply(W,1,sum)) - W
-    Wstar.eigen <- eigen(Wstar)
-    Wstar.val <- Wstar.eigen$values
-    det.Q.W <-  0.5 * sum(log((rho * Wstar.val + (1-rho))))    
+    if(!fix.rho.S) 
+    {
+        Wstar <- diag(apply(W,1,sum)) - W
+        Wstar.eigen <- eigen(Wstar)
+        Wstar.val <- Wstar.eigen$values
+        det.Q.W <-  0.5 * sum(log((rho * Wstar.val + (1-rho))))     
+    }else
+    {}
     
     
     #### Specify quantities that do not change
     offset.mat <- matrix(offset, nrow=K, ncol=N, byrow=FALSE) 
     regression.mat <- matrix(X.standardised %*% beta, nrow=K, ncol=N, byrow=FALSE)
     Y.mat <- matrix(Y, nrow=K, ncol=N, byrow=FALSE) 
+    Y.mat.miss <- matrix(Y.miss, nrow=K, ncol=N, byrow=FALSE)
     phi.mat <- matrix(phi, nrow=K, ncol=N, byrow=FALSE)   
     
     
-#### Beta update quantities
-    data.precision.beta <- t(X.standardised) %*% X.standardised
+    #### Beta update quantities
+    data.precision.beta <- t(X.short) %*% X.short
     if(length(prior.var.beta)==1)
     {
-    prior.precision.beta <- 1 / prior.var.beta
+        prior.precision.beta <- 1 / prior.var.beta
     }else
     {
-    prior.precision.beta <- solve(diag(prior.var.beta))
+        prior.precision.beta <- solve(diag(prior.var.beta))
     }
+    
+    
+    
+    ## Check for islands
+    W.list<- mat2listw(W)
+    W.nb <- W.list$neighbours
+    W.islands <- n.comp.nb(W.nb)
+    islands <- W.islands$comp.id
+    n.islands <- max(W.islands$nc)
+    if(rho==1 & gamma==1) 
+    {
+        tau2.phi.shape <- prior.tau2[1] + prior.tau2[1] + ((N-1) * (K-1))/2
+    }else if(rho==1)
+    {
+        tau2.phi.shape <- prior.tau2[1] + prior.tau2[1] + (N * (K-1))/2        
+    }else if(gamma==1)
+    {
+        tau2.phi.shape <- prior.tau2[1] + prior.tau2[1] + ((N-1) * K)/2          
+    }else
+    {}
+    
+    
 
 
     ###########################
@@ -226,7 +295,7 @@ nu2.shape <- prior.nu2[1] + N.all/2
     ## Start timer
     if(verbose)
     {
-        cat("Collecting", n.sample, "samples\n", sep = " ")
+        cat("Generating", n.sample, "samples\n", sep = " ")
         progressBar <- txtProgressBar(style = 3)
         percentage.points<-round((1:100/100)*n.sample)
     }else
@@ -241,7 +310,7 @@ nu2.shape <- prior.nu2[1] + N.all/2
         ##################
         ## Sample from nu2
         ##################
-        nu2.offset <- as.numeric(Y.mat - offset.mat - regression.mat - phi.mat)
+        nu2.offset <- as.numeric(Y.mat - offset.mat - regression.mat - phi.mat)[which.miss==1]
         nu2.scale <- prior.nu2[2]  + sum(nu2.offset^2)/2
         nu2 <- 1 / rgamma(1, nu2.shape, scale=(1/nu2.scale)) 
 
@@ -251,21 +320,22 @@ nu2.shape <- prior.nu2[1] + N.all/2
         ####################
         fc.precision <- prior.precision.beta + data.precision.beta / nu2
         fc.var <- solve(fc.precision)
-        beta.offset <- as.numeric(Y.mat - offset.mat - phi.mat)
-        beta.offset2 <- t(X.standardised) %*% beta.offset / nu2 + prior.precision.beta %*% prior.mean.beta
+        beta.offset <- as.numeric(Y.mat - offset.mat - phi.mat)[which.miss==1]
+        beta.offset2 <- t(X.short) %*% beta.offset / nu2 + prior.precision.beta %*% prior.mean.beta
         fc.mean <- fc.var %*% beta.offset2
         chol.var <- t(chol(fc.var))
         beta <- fc.mean + chol.var %*% rnorm(p)        
         regression.mat <- matrix(X.standardised %*% beta, nrow=K, ncol=N, byrow=FALSE)  
 
-               
+        
+        
         
         ####################
         ## Sample from phi
         ####################
-        phi.offset <- Y.mat - offset.mat - regression.mat
+        phi.offset <- Y.mat.miss - offset.mat - regression.mat
         den.offset <- rho * W.triplet.sum + 1 - rho
-        phi.temp <- gaussianarcarupdate(W.triplet, W.begfin, W.triplet.sum,  K, N, phi.mat, tau2, nu2, gamma, rho, phi.offset, den.offset)      
+        phi.temp <- gaussianarcarupdate(W.triplet, W.begfin, W.triplet.sum,  K, N, phi.mat, tau2, nu2, gamma, rho, phi.offset, den.offset, which.miss.mat)      
         phi <- as.numeric(phi.temp)  - mean(as.numeric(phi.temp))
         phi.mat <- matrix(phi, nrow=K, ncol=N, byrow=FALSE)
 
@@ -275,10 +345,15 @@ nu2.shape <- prior.nu2[1] + N.all/2
         ####################
         ## Sample from gamma
         ####################
+        if(!fix.rho.T)
+        {
         temp2 <- gammaquadformcompute(W.triplet, W.triplet.sum, W.n.triplet,  K, N, phi.mat, rho)
         mean.gamma <- temp2[[1]] / temp2[[2]]
         sd.gamma <- sqrt(tau2 / temp2[[2]])
         gamma <- rtrunc(n=1, spec="norm", a=0, b=1, mean=mean.gamma, sd=sd.gamma)
+        }else
+        {}
+        
         
         
         ####################
@@ -293,6 +368,8 @@ nu2.shape <- prior.nu2[1] + N.all/2
         ##################
         ## Sample from rho
         ##################
+        if(!fix.rho.S)
+        {
         proposal.rho <- rtrunc(n=1, spec="norm", a=0, b=1, mean=rho, sd=proposal.sd.rho)
         temp4 <- tauquadformcompute(W.triplet, W.triplet.sum, W.n.triplet,  K, N, phi.mat, proposal.rho, gamma)
         det.Q.W.proposal <- 0.5 * sum(log((proposal.rho * Wstar.val + (1-proposal.rho))))
@@ -308,7 +385,8 @@ nu2.shape <- prior.nu2[1] + N.all/2
         {
         }              
         accept[2] <- accept[2] + 1       
-
+        }else
+        {}
         
         
         #########################
@@ -316,7 +394,9 @@ nu2.shape <- prior.nu2[1] + N.all/2
         #########################
         fitted <- as.numeric(offset.mat + regression.mat + phi.mat)
         deviance.all <- dnorm(Y, mean = fitted, sd = rep(sqrt(nu2),N.all), log=TRUE)
-        deviance <- -2 * sum(deviance.all)          
+        like <- exp(deviance.all)
+        deviance <- -2 * sum(deviance.all, na.rm=TRUE)        
+
         
         
         ###################
@@ -327,12 +407,14 @@ nu2.shape <- prior.nu2[1] + N.all/2
             ele <- (j - burnin) / thin
             samples.beta[ele, ] <- beta
             samples.phi[ele, ] <- as.numeric(phi)
-            samples.rho[ele, ] <- rho
+            if(!fix.rho.S) samples.rho[ele, ] <- rho
+            if(!fix.rho.T) samples.gamma[ele, ] <- gamma
             samples.tau2[ele, ] <- tau2
             samples.nu2[ele, ] <- nu2
-            samples.gamma[ele, ] <- gamma
             samples.deviance[ele, ] <- deviance
             samples.fitted[ele, ] <- fitted
+            samples.like[ele, ] <- like
+            if(n.miss>0) samples.Y[ele, ] <- rnorm(n=n.miss, mean=fitted[which.miss==0], sd=sqrt(nu2))
         }else
         {
         }
@@ -347,19 +429,20 @@ nu2.shape <- prior.nu2[1] + N.all/2
         {
             #### Determine the acceptance probabilities
             accept.rho <- 100 * accept[1] / accept[2]
+            if(is.na(accept.rho)) accept.rho <- 45
             accept.all <- accept.all + accept
             accept <- rep(0,2)
                         
             #### rho tuning parameter
             if(accept.rho > 50)
             {
-                proposal.sd.rho <- 2 * min(proposal.sd.rho, 0.5)
+                proposal.sd.rho <- min(proposal.sd.rho + 0.1 * proposal.sd.rho, 0.5)
             }else if(accept.rho < 40)              
             {
-                proposal.sd.rho <- 0.5 * proposal.sd.rho
+                proposal.sd.rho <- proposal.sd.rho - 0.1 * proposal.sd.rho
             }else
             {
-            }      
+            }   
         }else
         {   
         }
@@ -388,11 +471,17 @@ nu2.shape <- prior.nu2[1] + N.all/2
 #### Summarise and save the results 
 ###################################
 ## Compute the acceptance rates
-accept.beta <- 100 * accept.all[1] / accept.all[2]
+    if(!fix.rho.S)
+    {
+        accept.rho <- 100 * accept.all[1] / accept.all[2]
+    }else
+    {
+        accept.rho <- NA    
+    }
 accept.phi <- 100
-accept.rho <- 100
-accept.final <- c(accept.beta, accept.phi, accept.rho)
-names(accept.final) <- c("beta", "phi", "rho")
+accept.beta <- 100
+accept.final <- c(accept.beta, accept.phi, accept.rho, 100)
+names(accept.final) <- c("beta", "phi", "rho.S", "rho.T")
     
     
 ## Compute information criterion (DIC, DIC3, WAIC)
@@ -401,18 +490,24 @@ regression.mat <- matrix(X.standardised %*% median.beta, nrow=K, ncol=N, byrow=F
 median.phi <- matrix(apply(samples.phi, 2, median), nrow=K, ncol=N)
 fitted.median <- as.numeric(offset.mat + median.phi + regression.mat)
 nu2.median <- median(samples.nu2)
-deviance.fitted <- -2 * sum(dnorm(Y, mean = fitted.median, sd = rep(sqrt(nu2.median),N.all), log = TRUE))
+deviance.fitted <- -2 * sum(dnorm(Y, mean = fitted.median, sd = rep(sqrt(nu2.median),N.all), log = TRUE), na.rm=TRUE)
 p.d <- median(samples.deviance) - deviance.fitted
 DIC <- 2 * median(samples.deviance) - deviance.fitted    
 
 
+#### Watanabe-Akaike Information Criterion (WAIC)
+LPPD <- sum(log(apply(samples.like,2,mean)), na.rm=TRUE)
+p.w <- sum(apply(log(samples.like),2,var), na.rm=TRUE)
+WAIC <- -2 * (LPPD - p.w)
+
+
 #### Compute the Conditional Predictive Ordinate  
 CPO <- rep(NA, N.all)
     for(j in 1:N.all)
     {
     CPO[j] <- 1/median((1 / dnorm(Y[j], mean=samples.fitted[ ,j], sd=sqrt(samples.nu2))))    
     }
-LMPL <- sum(log(CPO))   
+LMPL <- sum(log(CPO), na.rm=TRUE)   
     
     
 ## Create the Fitted values
@@ -459,23 +554,62 @@ colnames(summary.beta) <- c("Median", "2.5%", "97.5%", "n.sample", "% accept", "
 summary.hyper <- array(NA, c(4, 7))     
 summary.hyper[1,1:3] <- quantile(samples.tau2, c(0.5, 0.025, 0.975))
 summary.hyper[2,1:3] <- quantile(samples.nu2, c(0.5, 0.025, 0.975))
-summary.hyper[3,1:3] <- quantile(samples.rho, c(0.5, 0.025, 0.975))
-summary.hyper[4,1:3] <- quantile(samples.gamma, c(0.5, 0.025, 0.975))
-rownames(summary.hyper) <- c("tau2", "nu2", "rho", "gamma")     
+
+rownames(summary.hyper) <- c("tau2", "nu2", "rho.S", "rho.T")     
 summary.hyper[1, 4:7] <- c(n.keep, 100, effectiveSize(mcmc(samples.tau2)), geweke.diag(mcmc(samples.tau2))$z)     
 summary.hyper[2, 4:7] <- c(n.keep, 100, effectiveSize(mcmc(samples.nu2)), geweke.diag(mcmc(samples.nu2))$z)     
-summary.hyper[3, 4:7] <- c(n.keep, accept.rho, effectiveSize(mcmc(samples.rho)), geweke.diag(mcmc(samples.rho))$z)   
-summary.hyper[4, 4:7] <- c(n.keep, 100, effectiveSize(mcmc(samples.gamma)), geweke.diag(mcmc(samples.gamma))$z)       
-    
+
+if(!fix.rho.S)
+{
+    summary.hyper[3,1:3] <- quantile(samples.rho, c(0.5, 0.025, 0.975))
+    summary.hyper[3, 4:7] <- c(n.keep, accept.rho, effectiveSize(mcmc(samples.rho)), geweke.diag(mcmc(samples.rho))$z)  
+}else
+{
+    summary.hyper[3, 1:3] <- c(rho, rho, rho)
+    summary.hyper[3, 4:7] <- rep(NA, 4)
+}
+
+if(!fix.rho.T)
+{
+    summary.hyper[4,1:3] <- quantile(samples.gamma, c(0.5, 0.025, 0.975))
+    summary.hyper[4, 4:7] <- c(n.keep, 100, effectiveSize(mcmc(samples.gamma)), geweke.diag(mcmc(samples.gamma))$z)  
+}else
+{
+    summary.hyper[4, 1:3] <- c(gamma, gamma, gamma)
+    summary.hyper[4, 4:7] <- rep(NA, 4)
+}   
+
+
+
+
 summary.results <- rbind(summary.beta, summary.hyper)
 summary.results[ , 1:3] <- round(summary.results[ , 1:3], 4)
 summary.results[ , 4:7] <- round(summary.results[ , 4:7], 1)
     
     
 ## Compile and return the results
-modelfit <- c(DIC, p.d, LMPL)
-names(modelfit) <- c("DIC", "p.d", "LMPL")
-samples <- list(beta=mcmc(samples.beta.orig), phi=mcmc(samples.phi),  rho=mcmc(samples.rho), tau2=mcmc(samples.tau2), nu2=mcmc(samples.nu2), gamma=mcmc(samples.gamma), fitted=mcmc(samples.fitted))
+modelfit <- c(DIC, p.d, WAIC, p.w, LMPL)
+names(modelfit) <- c("DIC", "p.d", "WAIC", "p.w", "LMPL")
+if(fix.rho.S & fix.rho.T)
+{
+    samples.rhoext <- NA
+}else if(fix.rho.S & !fix.rho.T)
+{
+    samples.rhoext <- samples.gamma
+    names(samples.rhoext) <- "rho.T"
+}else if(!fix.rho.S & fix.rho.T)
+{
+    samples.rhoext <- samples.rho  
+    names(samples.rhoext) <- "rho.S"
+}else
+{
+    samples.rhoext <- cbind(samples.rho, samples.gamma)
+    colnames(samples.rhoext) <- c("rho.S", "rho.T")
+}
+if(n.miss==0) samples.Y = NA
+
+
+samples <- list(beta=mcmc(samples.beta.orig), phi=mcmc(samples.phi),  rho=mcmc(samples.rhoext), tau2=mcmc(samples.tau2), nu2=mcmc(samples.nu2), fitted=mcmc(samples.fitted), Y=mcmc(samples.Y))
 model.string <- c("Likelihood model - Gaussian (identity link function)", "\nLatent structure model - Autoregressive CAR model\n")
 results <- list(summary.results=summary.results, samples=samples, fitted.values=fitted.values, residuals=residuals, modelfit=modelfit, accept=accept.final, localised.structure=NULL, formula=formula, model=model.string,  X=X)
 class(results) <- "carbayesST"
