@@ -1,4 +1,4 @@
-gaussian.CARlinear <- function(formula, data=NULL, W, burnin, n.sample, thin=1,  prior.mean.beta=NULL, prior.var.beta=NULL, prior.mean.alpha=NULL, prior.var.alpha=NULL, prior.nu2=NULL, prior.tau2=NULL, verbose=TRUE)
+gaussian.CARlinear <- function(formula, data=NULL, W, burnin, n.sample, thin=1,  prior.mean.beta=NULL, prior.var.beta=NULL, prior.mean.alpha=NULL, prior.var.alpha=NULL, prior.nu2=NULL, prior.tau2=NULL, fix.rho.slo=FALSE, rho.slo=NULL, fix.rho.int=FALSE, rho.int=NULL,  verbose=TRUE)
 {
 #### Check on the verbose option
     if(is.null(verbose)) verbose=TRUE     
@@ -67,10 +67,30 @@ X.indicator <- rep(NA, p)       # To determine which parameter estimates to tran
     
     
     
+#### Format and check the neighbourhood matrix W
+if(!is.matrix(W)) stop("W is not a matrix.", call.=FALSE)
+K <- nrow(W)
+if(ncol(W)!= K) stop("W has the wrong number of columns.", call.=FALSE)
+if(nrow(W)!= K) stop("W has the wrong number of rows.", call.=FALSE)
+if(floor(N.all/K)!=ceiling(N.all/K)) stop("The number of spatial areas is not a multiple of the number of data points.", call.=FALSE)
+N <- N.all / K
+if(sum(is.na(W))>0) stop("W has missing 'NA' values.", call.=FALSE)
+if(!is.numeric(W)) stop("W has non-numeric values.", call.=FALSE)
+if(min(W)<0) stop("W has negative elements.", call.=FALSE)
+if(sum(W!=t(W))>0) stop("W is not symmetric.", call.=FALSE)
+if(min(apply(W, 1, sum))==0) stop("W has some areas with no neighbours (one of the row sums equals zero).", call.=FALSE)    
+
+
+
 #### Response variable
 Y <- model.response(frame)
-    if(sum(is.na(Y))>0) stop("the response has missing 'NA' values.", call.=FALSE)
-    if(!is.numeric(Y)) stop("the response variable has non-numeric values.", call.=FALSE)
+which.miss <- as.numeric(!is.na(Y))
+which.miss.mat <- matrix(which.miss, nrow=K, ncol=N, byrow=FALSE)
+n.miss <- N.all - sum(which.miss)
+Y.miss <- Y
+Y.miss[which.miss==0] <- median(Y, na.rm=TRUE)    
+if(!is.numeric(Y)) stop("the response variable has non-numeric values.", call.=FALSE)
+X.short <- X.standardised[which.miss==1, ]    
 
        
 #### Offset variable
@@ -82,37 +102,42 @@ offset <- try(model.offset(frame), silent=TRUE)
     
     
     
-#### Format and check the neighbourhood matrix W
-    if(!is.matrix(W)) stop("W is not a matrix.", call.=FALSE)
-K <- nrow(W)
-    if(ncol(W)!= K) stop("W has the wrong number of columns.", call.=FALSE)
-    if(nrow(W)!= K) stop("W has the wrong number of rows.", call.=FALSE)
-    if(floor(N.all/K)!=ceiling(N.all/K)) stop("The number of spatial areas is not a multiple of the number of data points.", call.=FALSE)
-N <- N.all / K
-    if(sum(is.na(W))>0) stop("W has missing 'NA' values.", call.=FALSE)
-    if(!is.numeric(W)) stop("W has non-numeric values.", call.=FALSE)
-    if(min(W)<0) stop("W has negative elements.", call.=FALSE)
-    if(sum(W!=t(W))>0) stop("W is not symmetric.", call.=FALSE)
-if(min(apply(W, 1, sum))==0) stop("W has some areas with no neighbours (one of the row sums equals zero).", call.=FALSE)    
-
-    
+  
     
 #### Specify the initial parameter values
 time <-(1:N - mean(1:N))/N
 time.all <- kronecker(time, rep(1,K))
-beta <- lm(Y~X.standardised-1 + time.all, offset=offset)$coefficients[1:p]
-alpha <- lm(Y~X.standardised-1 + time.all, offset=offset)$coefficients[(p+1)]
-regression.vec <- as.numeric(X.standardised %*% beta) + time.all * alpha
-res.temp <- Y - regression.vec - offset
+mod.glm <- glm(Y~X.standardised-1 + time.all, offset=offset)
+beta.mean <- mod.glm$coefficients
+beta.sd <- sqrt(diag(summary(mod.glm)$cov.scaled))
+temp <- rnorm(n=length(beta.mean), mean=beta.mean, sd=beta.sd)
+beta <- temp[1:p]
+alpha <- temp[(p+1)]
+
+res.temp <- Y - as.numeric(X.standardised %*% beta) - time.all * alpha - offset
+res.sd <- sd(res.temp, na.rm=TRUE)/5
+phi <- rnorm(n=K, mean=0, sd = res.sd)
+delta <- rnorm(n=K, mean=0, sd = res.sd)
+tau2.phi <- var(phi)/10
+tau2.delta <- var(delta)/10
+
+if(fix.rho.int)
+{
+    rho <- rho.int
+}else
+{
+    rho <- runif(1)       
+}
+
+if(fix.rho.slo)
+{
+    lambda <- rho.slo
+}else
+{
+    lambda <- runif(1)       
+}   
+
 
-phi <- rnorm(n=K, mean=0, sd = 2*sd(res.temp))
-delta <- rnorm(n=K, mean=0, sd = 2*sd(res.temp))
-tau2.phi <- runif(1, min=var(res.temp)/2, max=var(res.temp)*2)
-tau2.delta <- runif(1, min=var(res.temp)/2, max=var(res.temp)*2)
-nu2.phi <- runif(1, min=var(res.temp)/2, max=var(res.temp)*2)
-rho <- runif(1)  
-lambda <- runif(1)  
-    
     
     
 #### Check and specify the priors
@@ -168,20 +193,38 @@ lambda <- runif(1)
     
     
 
+## Check for errors on rho and fix.rho
+if(!is.logical(fix.rho.int)) stop("fix.rho.int is not logical.", call.=FALSE)   
+if(fix.rho.int & is.null(rho.int)) stop("rho.int is fixed but an initial value was not set.", call.=FALSE)   
+if(fix.rho.int & !is.numeric(rho.int) ) stop("rho.S is not numeric.", call.=FALSE)  
+if(rho<0 ) stop("rho.int is outside the range [0, 1].", call.=FALSE)  
+if(rho>1 ) stop("rho.int is outside the range [0, 1].", call.=FALSE)  
+
+## Check for errors on rho and fix.rho
+if(!is.logical(fix.rho.slo)) stop("fix.rho.slo is not logical.", call.=FALSE)   
+if(fix.rho.slo & is.null(rho.slo)) stop("rho.slo is fixed but an initial value was not set.", call.=FALSE)   
+if(fix.rho.slo & !is.numeric(rho.slo) ) stop("rho.slo is not numeric.", call.=FALSE)  
+if(lambda<0 ) stop("rho.slo is outside the range [0, 1].", call.=FALSE)  
+if(lambda>1 ) stop("rho.slo is outside the range [0, 1].", call.=FALSE)  
+
+
+
 #### Set up matrices to store samples
 n.keep <- floor((n.sample - burnin)/thin)
 samples.beta <- array(NA, c(n.keep, p))
 samples.alpha <- array(NA, c(n.keep, 1))
 samples.phi <- array(NA, c(n.keep, K))
 samples.delta <- array(NA, c(n.keep, K))
-samples.rho <- array(NA, c(n.keep, 1))
-samples.lambda <- array(NA, c(n.keep, 1))
+if(!fix.rho.int) samples.rho <- array(NA, c(n.keep, 1))
+if(!fix.rho.slo) samples.lambda <- array(NA, c(n.keep, 1))
 samples.nu2 <- array(NA, c(n.keep, 1))
 samples.tau2 <- array(NA, c(n.keep, 2))
-colnames(samples.tau2) <- c("tau2.phi", "tau2.delta")
+colnames(samples.tau2) <- c("tau2.int", "tau2.slo")
 samples.fitted <- array(NA, c(n.keep, N.all))
+samples.like <- array(NA, c(n.keep, N.all))
 samples.deviance <- array(NA, c(n.keep, 1))
-    
+if(n.miss>0) samples.Y <- array(NA, c(n.keep, n.miss))
+
     
     
 #### Specify the Metropolis quantities
@@ -228,20 +271,34 @@ temp <- 1
 Wstar <- diag(apply(W,1,sum)) - W
 Wstar.eigen <- eigen(Wstar)
 Wstar.val <- Wstar.eigen$values
-det.Q.rho <-  0.5 * sum(log((rho * Wstar.val + (1-rho))))    
-det.Q.lambda <-  0.5 * sum(log((lambda * Wstar.val + (1-lambda))))      
+if(!fix.rho.int) det.Q.rho <-  0.5 * sum(log((rho * Wstar.val + (1-rho))))    
+if(!fix.rho.slo) det.Q.lambda <-  0.5 * sum(log((lambda * Wstar.val + (1-lambda))))     
     
     
 #### Specify quantities that do not change
 offset.mat <- matrix(offset, nrow=K, ncol=N, byrow=FALSE) 
 regression.mat <- matrix(X.standardised %*% beta, nrow=K, ncol=N, byrow=FALSE)   
 Y.mat <- matrix(Y, nrow=K, ncol=N, byrow=FALSE)
+Y.mat.miss <- matrix(Y.miss, nrow=K, ncol=N, byrow=FALSE)
 phi.mat <- matrix(rep(phi, N), byrow=F, nrow=K)
 time.mat <- matrix(rep(time, K), byrow=TRUE, nrow=K)    
 delta.time.mat <- apply(time.mat, 2, "*", delta)
-    
+alpha.offset1 <- sum(time.mat^2 * which.miss.mat)
+ntime.miss <- apply(which.miss.mat,1,sum)
+ntime.miss2 <- apply(which.miss.mat * time.mat^2,1,sum)
+
+## Check for islands
+W.list<- mat2listw(W)
+W.nb <- W.list$neighbours
+W.islands <- n.comp.nb(W.nb)
+islands <- W.islands$comp.id
+n.islands <- max(W.islands$nc)
+if(rho==1) tau2.phi.shape <- prior.tau2[1] + 0.5 * (K-n.islands)   
+if(lambda==1) tau2.delta.shape <- prior.tau2[1] + 0.5 * (K-n.islands)     
+
+
 #### Beta update quantities
-data.precision.beta <- t(X.standardised) %*% X.standardised
+data.precision.beta <- t(X.short) %*% X.short
     if(length(prior.var.beta)==1)
     {
     prior.precision.beta <- 1 / prior.var.beta
@@ -258,7 +315,7 @@ data.precision.beta <- t(X.standardised) %*% X.standardised
 ## Start timer
     if(verbose)
     {
-        cat("Collecting", n.sample, "samples\n", sep = " ")
+        cat("Generating", n.sample, "samples\n", sep = " ")
         progressBar <- txtProgressBar(style = 3)
         percentage.points<-round((1:100/100)*n.sample)
     }else
@@ -272,7 +329,7 @@ data.precision.beta <- t(X.standardised) %*% X.standardised
     ##################
     ## Sample from nu2
     ##################
-    nu2.offset <- as.numeric(Y.mat - offset.mat - regression.mat - phi.mat - delta.time.mat - alpha * time.mat)
+    nu2.offset <- as.numeric(Y.mat - offset.mat - regression.mat - phi.mat - delta.time.mat - alpha * time.mat)[which.miss==1]
     nu2.scale <- prior.nu2[2]  + sum(nu2.offset^2)/2
     nu2 <- 1 / rgamma(1, nu2.shape, scale=(1/nu2.scale)) 
 
@@ -283,8 +340,8 @@ data.precision.beta <- t(X.standardised) %*% X.standardised
     ####################
     fc.precision <- prior.precision.beta + data.precision.beta / nu2
     fc.var <- solve(fc.precision)
-    beta.offset <- as.numeric(Y.mat - offset.mat - phi.mat - delta.time.mat - alpha * time.mat)
-    beta.offset2 <- t(X.standardised) %*% beta.offset / nu2 + prior.precision.beta %*% prior.mean.beta
+    beta.offset <- as.numeric(Y.mat - offset.mat - phi.mat - delta.time.mat - alpha * time.mat)[which.miss==1]
+    beta.offset2 <- t(X.short) %*% beta.offset / nu2 + prior.precision.beta %*% prior.mean.beta
     fc.mean <- fc.var %*% beta.offset2
     chol.var <- t(chol(fc.var))
     beta <- fc.mean + chol.var %*% rnorm(p)        
@@ -295,10 +352,10 @@ data.precision.beta <- t(X.standardised) %*% X.standardised
     ####################
     ## Sample from alpha
     ####################
-    fc.var <- 1 / (1 / prior.var.alpha + K * sum(time^2) / nu2)
-    alpha.offset <- Y.mat - offset.mat - regression.mat - phi.mat - delta.time.mat
-    alpha.offset2 <- sum(alpha.offset / (nu2 / time.mat)) +  prior.mean.alpha / prior.var.alpha
-    fc.mean <- fc.var * alpha.offset2
+    fc.var <- 1 / (1 / prior.var.alpha + alpha.offset1 / nu2)
+    alpha.offset <- (Y.mat - offset.mat - regression.mat - phi.mat - delta.time.mat) * time.mat
+    alpha.offset2 <- sum(alpha.offset, na.rm=TRUE) / nu2
+    fc.mean <- fc.var * (alpha.offset2 +  prior.mean.alpha / prior.var.alpha)
     alpha <- rnorm(n=1, mean=fc.mean, sd=sqrt(fc.var))
          
       
@@ -307,10 +364,16 @@ data.precision.beta <- t(X.standardised) %*% X.standardised
     ## Sample from phi
     ####################
     phi.offset <- Y.mat - offset.mat - regression.mat - delta.time.mat - alpha * time.mat
-    phi.offset2 <- apply(phi.offset,1, sum)
-    temp1 <- gaussiancarupdate(W.triplet, W.begfin, W.triplet.sum, K, phi, tau2.phi, nu2, phi.offset2, rho, N)
+    phi.offset2 <- apply(phi.offset,1, sum, na.rm=TRUE)
+    temp1 <- gaussiancarupdate(W.triplet, W.begfin, W.triplet.sum, K, phi, tau2.phi, nu2, phi.offset2, rho, ntime.miss)
     phi <- temp1
-    phi <- phi - mean(phi)
+    if(rho<1)
+    {
+        phi <- phi - mean(phi)
+    }else
+    {
+        phi[which(islands==1)] <- phi[which(islands==1)] - mean(phi[which(islands==1)])   
+    }
     phi.mat <- matrix(rep(phi, N), byrow=F, nrow=K)    
 
         
@@ -319,13 +382,17 @@ data.precision.beta <- t(X.standardised) %*% X.standardised
     ####################
     ## Sample from delta
     ####################
-    delta.offset <- Y.mat - offset.mat - regression.mat - phi.mat - alpha * time.mat
-    delta.offset2 <- delta.offset * time.mat    
-    delta.offset3 <- apply(delta.offset2,1, sum)
-    delta.offset4 <- sum(time^2)
-    temp2 <- gaussiancarupdate(W.triplet, W.begfin, W.triplet.sum, K, delta, tau2.delta, nu2, delta.offset3, lambda, delta.offset4)
+    delta.offset <- (Y.mat - offset.mat - regression.mat - phi.mat - alpha * time.mat) * time.mat
+    delta.offset2 <- apply(delta.offset,1, sum, na.rm=TRUE)
+    temp2 <- gaussiancarupdate(W.triplet, W.begfin, W.triplet.sum, K, delta, tau2.delta, nu2, delta.offset2, lambda, ntime.miss2)
     delta <- temp2
-    delta <- delta - mean(delta)
+    if(lambda <1)
+    {
+        delta <- delta - mean(delta)
+    }else
+    {
+        delta[which(islands==1)] <- delta[which(islands==1)] - mean(delta[which(islands==1)])   
+    }
     delta.time.mat <- apply(time.mat, 2, "*", delta)
          
         
@@ -351,6 +418,8 @@ data.precision.beta <- t(X.standardised) %*% X.standardised
     ##################
     ## Sample from rho
     ##################
+    if(!fix.rho.int)
+    {
     proposal.rho <- rtrunc(n=1, spec="norm", a=0, b=1, mean=rho, sd=proposal.sd.rho)   
     temp3 <- quadform(W.triplet, W.triplet.sum, W.n.triplet, K, phi, phi, proposal.rho)
     det.Q.proposal <- 0.5 * sum(log((proposal.rho * Wstar.val + (1-proposal.rho))))              
@@ -368,12 +437,16 @@ data.precision.beta <- t(X.standardised) %*% X.standardised
         {
         }              
         accept[2] <- accept[2] + 1           
+    }else
+    {}
         
-        
-        
+     
+       
     #####################
     ## Sample from lambda
     #####################
+    if(!fix.rho.slo)
+    {
     proposal.lambda <- rtrunc(n=1, spec="norm", a=0, b=1, mean=lambda, sd=proposal.sd.lambda)   
     temp3 <- quadform(W.triplet, W.triplet.sum, W.n.triplet, K, delta, delta, proposal.lambda)
     det.Q.proposal <- 0.5 * sum(log((proposal.lambda * Wstar.val + (1-proposal.lambda))))              
@@ -391,17 +464,21 @@ data.precision.beta <- t(X.standardised) %*% X.standardised
         {
         }              
         accept[4] <- accept[4] + 1           
+    }else
+    {}
         
-        
-        
+     
+       
     #########################
     ## Calculate the deviance
     #########################
     fitted <- as.numeric(offset.mat + regression.mat + phi.mat + delta.time.mat + alpha * time.mat)
     deviance.all <- dnorm(Y, mean = fitted, sd = rep(sqrt(nu2),N.all), log=TRUE)
-    deviance <- -2 * sum(deviance.all)  
-        
+    like <- exp(deviance.all)
+    deviance <- -2 * sum(deviance.all, na.rm=TRUE)      
         
+    
+    
     ###################
     ## Save the results
     ###################
@@ -412,12 +489,15 @@ data.precision.beta <- t(X.standardised) %*% X.standardised
             samples.phi[ele, ] <- phi
             samples.delta[ele, ] <- delta
             samples.alpha[ele, ] <- alpha
-            samples.rho[ele, ] <- rho
-            samples.lambda[ele, ] <- lambda
+            if(!fix.rho.int) samples.rho[ele, ] <- rho
+            if(!fix.rho.slo) samples.lambda[ele, ] <- lambda
             samples.nu2[ele, ] <- nu2
             samples.tau2[ele, ] <- c(tau2.phi, tau2.delta)
             samples.deviance[ele, ] <- deviance
             samples.fitted[ele, ] <- fitted
+            samples.like[ele, ] <- like
+            if(n.miss>0) samples.Y[ele, ] <- rnorm(n=n.miss, mean=fitted[which.miss==0], sd=sqrt(nu2))
+            
         }else
         {
         }
@@ -431,30 +511,32 @@ data.precision.beta <- t(X.standardised) %*% X.standardised
         {
         #### Determine the acceptance probabilities
         accept.rho <- 100 * accept[1] / accept[2]
+        if(is.na(accept.rho)) accept.rho <- 45
         accept.lambda <- 100 * accept[3] / accept[4]
+        if(is.na(accept.lambda)) accept.lambda <- 45
         accept.all <- accept.all + accept
         accept <- rep(0,4)
             
-            #### rho tuning parameter
-            if(accept.rho > 50)
-            {
-                proposal.sd.rho <- min(2 * proposal.sd.rho, 0.5)
-            }else if(accept.rho < 40)              
-            {
-                proposal.sd.rho <- 0.5 * proposal.sd.rho
-            }else
-            {
-            }
-            #### lambda tuning parameter
-            if(accept.lambda > 50)
-            {
-                proposal.sd.lambda <- min(2 * proposal.sd.lambda, 0.5)
-            }else if(accept.lambda < 40)              
-            {
-                proposal.sd.lambda <- 0.5 * proposal.sd.lambda
-            }else
-            {
-            }      
+        #### rho tuning parameter
+        if(accept.rho > 50)
+        {
+            proposal.sd.rho <- min(proposal.sd.rho + 0.1 * proposal.sd.rho, 0.5)
+        }else if(accept.rho < 40)              
+        {
+            proposal.sd.rho <- proposal.sd.rho - 0.1 * proposal.sd.rho
+        }else
+        {
+        }
+        #### lambda tuning parameter
+        if(accept.lambda > 50)
+        {
+            proposal.sd.lambda <- min(proposal.sd.lambda + 0.1 * proposal.sd.lambda, 0.5)
+        }else if(accept.lambda < 40)              
+        {
+            proposal.sd.lambda <- proposal.sd.lambda - 0.1 * proposal.sd.lambda
+        }else
+        {
+        }    
         }else
         {   
         }
@@ -482,10 +564,23 @@ data.precision.beta <- t(X.standardised) %*% X.standardised
 #### Summarise and save the results 
 ###################################
 ## Compute the acceptance rates
-accept.rho <- 100 * accept.all[1] / accept.all[2]
-accept.lambda <- 100 * accept.all[3] / accept.all[4]
+if(!fix.rho.int)
+{
+    accept.rho <- 100 * accept.all[1] / accept.all[2]
+}else
+{
+    accept.rho <- NA    
+}
+
+if(!fix.rho.slo)
+{
+    accept.lambda <- 100 * accept.all[3] / accept.all[4]
+}else
+{
+    accept.lambda <- NA    
+}
 accept.final <- c(rep(100,4), accept.rho, accept.lambda)
-names(accept.final) <- c("beta", "alpha", "phi", "delta", "rho", "lambda")
+names(accept.final) <- c("beta", "alpha", "phi", "delta", "rho.int", "rho.slo")
     
       
 ## Compute DIC
@@ -499,18 +594,26 @@ regression.mat <- matrix(X.standardised %*% median.beta, nrow=K, ncol=N, byrow=F
 LP <- offset.mat + regression.mat + phi.mat + delta.time.mat + median.alpha * time.mat
 fitted.median <- as.numeric(LP)
 nu2.median <- median(samples.nu2)
-deviance.fitted <- -2 * sum(dnorm(Y, mean = fitted.median, sd = rep(sqrt(nu2.median),N.all), log = TRUE))
+deviance.fitted <- -2 * sum(dnorm(Y, mean = fitted.median, sd = rep(sqrt(nu2.median),N.all), log = TRUE), na.rm=TRUE)
 p.d <- median(samples.deviance) - deviance.fitted
 DIC <- 2 * median(samples.deviance) - deviance.fitted    
 
 
+
+#### Watanabe-Akaike Information Criterion (WAIC)
+LPPD <- sum(log(apply(samples.like,2,mean)), na.rm=TRUE)
+p.w <- sum(apply(log(samples.like),2,var), na.rm=TRUE)
+WAIC <- -2 * (LPPD - p.w)
+
+
+
 #### Compute the Conditional Predictive Ordinate  
 CPO <- rep(NA, N.all)
     for(j in 1:N.all)
     {
     CPO[j] <- 1/median((1 / dnorm(Y[j], mean=samples.fitted[ ,j], sd=sqrt(samples.nu2))))    
     }
-LMPL <- sum(log(CPO))  
+LMPL <- sum(log(CPO), na.rm=TRUE)  
 
   
 ## Create the Fitted values
@@ -559,26 +662,63 @@ summary.hyper[1,1:3] <- quantile(samples.alpha, c(0.5, 0.025, 0.975))
 summary.hyper[2,1:3] <- quantile(samples.tau2[ ,1], c(0.5, 0.025, 0.975))
 summary.hyper[3,1:3] <- quantile(samples.tau2[ ,2], c(0.5, 0.025, 0.975))
 summary.hyper[4,1:3] <- quantile(samples.nu2, c(0.5, 0.025, 0.975))
-summary.hyper[5,1:3] <- quantile(samples.rho, c(0.5, 0.025, 0.975))
-summary.hyper[6,1:3] <- quantile(samples.lambda, c(0.5, 0.025, 0.975))
-rownames(summary.hyper) <- c("alpha", "tau2.phi", "tau2.delta",  "nu2", "rho.phi", "rho.delta")     
+rownames(summary.hyper) <- c("alpha", "tau2.int", "tau2.slo",  "nu2", "rho.int", "rho.slo")   
+
 summary.hyper[1, 4:7] <- c(n.keep, 100, effectiveSize(mcmc(samples.alpha)), geweke.diag(mcmc(samples.alpha))$z)     
 summary.hyper[2, 4:7] <- c(n.keep, 100, effectiveSize(mcmc(samples.tau2[ ,1])), geweke.diag(mcmc(samples.tau2[ ,1]))$z)   
 summary.hyper[3, 4:7] <- c(n.keep, 100, effectiveSize(mcmc(samples.tau2[ ,2])), geweke.diag(mcmc(samples.tau2[ ,2]))$z)   
 summary.hyper[4, 4:7] <- c(n.keep, 100, effectiveSize(mcmc(samples.nu2)), geweke.diag(mcmc(samples.nu2))$z)   
-summary.hyper[5, 4:7] <- c(n.keep, accept.rho, effectiveSize(mcmc(samples.rho)), geweke.diag(mcmc(samples.rho))$z)   
-summary.hyper[6, 4:7] <- c(n.keep, accept.lambda, effectiveSize(mcmc(samples.lambda)), geweke.diag(mcmc(samples.lambda))$z)         
-    
+
+if(!fix.rho.int)
+{
+    summary.hyper[5, 1:3] <- quantile(samples.rho, c(0.5, 0.025, 0.975))
+    summary.hyper[5, 4:7] <- c(n.keep, accept.rho, effectiveSize(samples.rho), geweke.diag(samples.rho)$z)
+}else
+{
+    summary.hyper[5, 1:3] <- c(rho, rho, rho)
+    summary.hyper[5, 4:7] <- rep(NA, 4)
+}
+
+if(!fix.rho.slo)
+{
+    summary.hyper[6, 1:3] <- quantile(samples.lambda, c(0.5, 0.025, 0.975))
+    summary.hyper[6, 4:7] <- c(n.keep, accept.lambda, effectiveSize(samples.lambda), geweke.diag(samples.lambda)$z)
+}else
+{
+    summary.hyper[6, 1:3] <- c(lambda, lambda, lambda)
+    summary.hyper[6, 4:7] <- rep(NA, 4)
+}   
+
+
 summary.results <- rbind(summary.beta, summary.hyper)
 summary.results[ , 1:3] <- round(summary.results[ , 1:3], 4)
 summary.results[ , 4:7] <- round(summary.results[ , 4:7], 1)
     
     
 ## Compile and return the results
-modelfit <- c(DIC, p.d, LMPL)
-names(modelfit) <- c("DIC", "p.d", "LMPL")
-samples.rhoext <- cbind(samples.rho, samples.lambda)
-samples <- list(beta=mcmc(samples.beta.orig), alpha=mcmc(samples.alpha), phi=mcmc(samples.phi),  delta=mcmc(samples.delta), tau2=mcmc(samples.tau2), nu2=mcmc(samples.nu2), rho=mcmc(samples.rhoext), fitted=mcmc(samples.fitted))        
+modelfit <- c(DIC, p.d, WAIC, p.w, LMPL)
+names(modelfit) <- c("DIC", "p.d", "WAIC", "p.w", "LMPL")
+
+if(fix.rho.int & fix.rho.slo)
+{
+    samples.rhoext <- NA
+}else if(fix.rho.int & !fix.rho.slo)
+{
+    samples.rhoext <- samples.lambda
+    names(samples.rhoext) <- "rho.slo"
+}else if(!fix.rho.int & fix.rho.slo)
+{
+    samples.rhoext <- samples.rho  
+    names(samples.rhoext) <- "rho.int"
+}else
+{
+    samples.rhoext <- cbind(samples.rho, samples.lambda)
+    colnames(samples.rhoext) <- c("rho.int", "rho.slo")
+}
+
+if(n.miss==0) samples.Y = NA
+
+samples <- list(beta=mcmc(samples.beta.orig), alpha=mcmc(samples.alpha), phi=mcmc(samples.phi),  delta=mcmc(samples.delta), tau2=mcmc(samples.tau2), nu2=mcmc(samples.nu2), rho=mcmc(samples.rhoext), fitted=mcmc(samples.fitted), Y=mcmc(samples.Y))        
 model.string <- c("Likelihood model - Gaussian (identity link function)", "\nLatent structure model - Spatially autocorrelated linear time trends\n")
 results <- list(summary.results=summary.results, samples=samples, fitted.values=fitted.values, residuals=residuals, modelfit=modelfit, accept=accept.final, localised.structure=NULL, formula=formula, model=model.string,  X=X)
 class(results) <- "carbayesST"
